@@ -1,8 +1,5 @@
 /* 
-
-   bmc-argp.h - function prototypes
-
-   Copyright (C) 2006 FreeIPMI Core Team
+   Copyright (C) 2003-2008 FreeIPMI Core Team
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -25,8 +22,6 @@
 
 #include "bmc-config.h"
 
-void bmc_config_argp_parse (int argc, char *argv[], struct config_arguments *args);
-
-int bmc_config_args_validate (struct config_arguments *args);
+void bmc_config_argp_parse (int argc, char *argv[], struct bmc_config_arguments *args);
 
 #endif /* _BMC_ARGP_H_ */
