@@ -1,6 +1,24 @@
+/* 
+   Copyright (C) 2003-2008 FreeIPMI Core Team
+
+   This program is free software; you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2, or (at your option)
+   any later version.
+
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
+
+   You should have received a copy of the GNU General Public License
+   along with this program; if not, write to the Free Software Foundation,
+   Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.  
+*/
+
 #if HAVE_CONFIG_H
 #include "config.h"
-#endif
+#endif /* HAVE_CONFIG_H */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -13,7 +31,11 @@
 #include "bmc-config-validate.h"
 #include "bmc-config-utils.h"
 
-/* convenience struct */
+#include "freeipmi-portability.h"
+#include "pstdout.h"
+#include "tool-fiid-wrappers.h"
+
+/* convenience structs */
 struct bmc_authentication_level {
   uint8_t callback_level_none;
   uint8_t callback_level_md2;
@@ -42,6 +64,64 @@ struct bmc_authentication_level {
   uint8_t oem_level_oem_proprietary;
 };
 
+static config_err_t 
+_get_authentication_type_support (bmc_config_state_data_t *state_data)
+{
+  fiid_obj_t obj_cmd_rs = NULL;
+  uint64_t val;
+  config_err_t rv = CONFIG_ERR_FATAL_ERROR;
+  config_err_t ret;
+  uint8_t channel_number;
+  
+  assert(state_data);
+  
+  _FIID_OBJ_CREATE(obj_cmd_rs, tmpl_cmd_get_lan_configuration_parameters_authentication_type_support_rs);
+  
+  if ((ret = get_lan_channel_number (state_data, &channel_number)) != CONFIG_ERR_SUCCESS)
+    {
+      rv = ret;
+      goto cleanup;
+    }
+
+  if (ipmi_cmd_get_lan_configuration_parameters_authentication_type_support (state_data->ipmi_ctx,
+                                                                             channel_number,
+                                                                             IPMI_GET_LAN_PARAMETER,
+                                                                             SET_SELECTOR,
+                                                                             BLOCK_SELECTOR,
+                                                                             obj_cmd_rs) < 0)
+    {
+      if (state_data->prog_data->args->config_args.common.debug)
+        pstdout_fprintf(state_data->pstate,
+                        stderr,
+                        "ipmi_cmd_get_lan_configuration_parameters_authentication_type_support: %s\n",
+                        ipmi_ctx_strerror(ipmi_ctx_errnum(state_data->ipmi_ctx)));
+      if (!IPMI_CTX_ERRNUM_IS_FATAL_ERROR(state_data->ipmi_ctx))
+        rv = CONFIG_ERR_NON_FATAL_ERROR;
+      goto cleanup;
+    }
+
+  _FIID_OBJ_GET (obj_cmd_rs, "none", &val);
+  state_data->authentication_type_none = val;
+  
+  _FIID_OBJ_GET (obj_cmd_rs, "md2", &val);
+  state_data->authentication_type_md2 = val;
+
+  _FIID_OBJ_GET (obj_cmd_rs, "md5", &val);
+  state_data->authentication_type_md5 = val;
+  
+  _FIID_OBJ_GET (obj_cmd_rs, "straight_password", &val);
+  state_data->authentication_type_straight_password = val;
+  
+  _FIID_OBJ_GET (obj_cmd_rs, "oem_proprietary", &val);
+  state_data->authentication_type_oem_proprietary = val;
+  
+  state_data->authentication_type_initialized++;
+  rv = CONFIG_ERR_SUCCESS;
+ cleanup:
+  _FIID_OBJ_DESTROY(obj_cmd_rs);
+  return (rv); 
+}
+
 static config_err_t 
 _get_authentication_type_enables (bmc_config_state_data_t *state_data,
                                   struct bmc_authentication_level *al)
@@ -55,8 +135,7 @@ _get_authentication_type_enables (bmc_config_state_data_t *state_data,
   assert(state_data);
   assert(al);
 
-  if (!(obj_cmd_rs = Fiid_obj_create(tmpl_cmd_get_lan_configuration_parameters_authentication_type_enables_rs)))
-    goto cleanup;
+  _FIID_OBJ_CREATE(obj_cmd_rs, tmpl_cmd_get_lan_configuration_parameters_authentication_type_enables_rs);
   
   if ((ret = get_lan_channel_number (state_data, &channel_number)) != CONFIG_ERR_SUCCESS)
     {
@@ -64,124 +143,101 @@ _get_authentication_type_enables (bmc_config_state_data_t *state_data,
       goto cleanup;
     }
 
-  if (ipmi_cmd_get_lan_configuration_parameters_authentication_type_enables (state_data->dev, 
+  if (ipmi_cmd_get_lan_configuration_parameters_authentication_type_enables (state_data->ipmi_ctx, 
 									     channel_number, 
 									     IPMI_GET_LAN_PARAMETER, 
 									     SET_SELECTOR, 
 									     BLOCK_SELECTOR, 
 									     obj_cmd_rs) < 0)
     {
-      if (state_data->prog_data->args->common.flags & IPMI_FLAGS_DEBUG_DUMP)
-        fprintf(stderr,
-                "ipmi_cmd_get_lan_configuration_parameters_authentication_type_enables: %s\n",
-                ipmi_device_strerror(ipmi_device_errnum(state_data->dev)));
-      rv = CONFIG_ERR_NON_FATAL_ERROR;
+      if (state_data->prog_data->args->config_args.common.debug)
+        pstdout_fprintf(state_data->pstate,
+                        stderr,
+                        "ipmi_cmd_get_lan_configuration_parameters_authentication_type_enables: %s\n",
+                        ipmi_ctx_strerror(ipmi_ctx_errnum(state_data->ipmi_ctx)));
+      if (!IPMI_CTX_ERRNUM_IS_FATAL_ERROR(state_data->ipmi_ctx))
+        rv = CONFIG_ERR_NON_FATAL_ERROR;
       goto cleanup;
     }
   
-  if (Fiid_obj_get (obj_cmd_rs, "callback_level.none", &val) < 0)
-    goto cleanup;
+  _FIID_OBJ_GET (obj_cmd_rs, "callback_level.none", &val);
   al->callback_level_none = val;
   
-  if (Fiid_obj_get (obj_cmd_rs, "callback_level.md2", &val) < 0)
-    goto cleanup;
+  _FIID_OBJ_GET (obj_cmd_rs, "callback_level.md2", &val);
   al->callback_level_md2 = val;
   
-  if (Fiid_obj_get (obj_cmd_rs, "callback_level.md5", &val) < 0)
-    goto cleanup;
+  _FIID_OBJ_GET (obj_cmd_rs, "callback_level.md5", &val);
   al->callback_level_md5 = val;
   
-  if (Fiid_obj_get (obj_cmd_rs, "callback_level.straight_password", &val) < 0)
-    goto cleanup;
+  _FIID_OBJ_GET (obj_cmd_rs, "callback_level.straight_password", &val);
   al->callback_level_straight_password = val;
   
-  if (Fiid_obj_get (obj_cmd_rs, "callback_level.oem_proprietary", &val) < 0)
-    goto cleanup;
+  _FIID_OBJ_GET (obj_cmd_rs, "callback_level.oem_proprietary", &val);
   al->callback_level_oem_proprietary = val;
   
-  if (Fiid_obj_get (obj_cmd_rs, "user_level.none", &val) < 0)
-    goto cleanup;
+  _FIID_OBJ_GET (obj_cmd_rs, "user_level.none", &val);
   al->user_level_none = val;
   
-  if (Fiid_obj_get (obj_cmd_rs, "user_level.md2", &val) < 0)
-    goto cleanup;
+  _FIID_OBJ_GET (obj_cmd_rs, "user_level.md2", &val);
   al->user_level_md2 = val;
   
-  if (Fiid_obj_get (obj_cmd_rs, "user_level.md5", &val) < 0)
-    goto cleanup;
+  _FIID_OBJ_GET (obj_cmd_rs, "user_level.md5", &val);
   al->user_level_md5 = val;
   
-  if (Fiid_obj_get (obj_cmd_rs, "user_level.straight_password", &val) < 0)
-    goto cleanup;
+  _FIID_OBJ_GET (obj_cmd_rs, "user_level.straight_password", &val);
   al->user_level_straight_password = val;
   
-  if (Fiid_obj_get (obj_cmd_rs, "user_level.oem_proprietary", &val) < 0)
-    goto cleanup;
+  _FIID_OBJ_GET (obj_cmd_rs, "user_level.oem_proprietary", &val);
   al->user_level_oem_proprietary = val;
   
-  if (Fiid_obj_get (obj_cmd_rs, "operator_level.none", &val) < 0)
-    goto cleanup;
+  _FIID_OBJ_GET (obj_cmd_rs, "operator_level.none", &val);
   al->operator_level_none = val;
   
-  if (Fiid_obj_get (obj_cmd_rs, "operator_level.md2", &val) < 0)
-    goto cleanup;
+  _FIID_OBJ_GET (obj_cmd_rs, "operator_level.md2", &val);
   al->operator_level_md2 = val;
   
-  if (Fiid_obj_get (obj_cmd_rs, "operator_level.md5", &val) < 0)
-    goto cleanup;
+  _FIID_OBJ_GET (obj_cmd_rs, "operator_level.md5", &val);
   al->operator_level_md5 = val;
   
-  if (Fiid_obj_get (obj_cmd_rs, "operator_level.straight_password", &val) < 0)
-    goto cleanup;
+  _FIID_OBJ_GET (obj_cmd_rs, "operator_level.straight_password", &val);
   al->operator_level_straight_password = val;
   
-  if (Fiid_obj_get (obj_cmd_rs, "operator_level.oem_proprietary", &val) < 0)
-    goto cleanup;
+  _FIID_OBJ_GET (obj_cmd_rs, "operator_level.oem_proprietary", &val);
   al->operator_level_oem_proprietary = val;
   
-  if (Fiid_obj_get (obj_cmd_rs, "admin_level.none", &val) < 0)
-    goto cleanup;
+  _FIID_OBJ_GET (obj_cmd_rs, "admin_level.none", &val);
   al->admin_level_none = val;
   
-  if (Fiid_obj_get (obj_cmd_rs, "admin_level.md2", &val) < 0)
-    goto cleanup;
+  _FIID_OBJ_GET (obj_cmd_rs, "admin_level.md2", &val);
   al->admin_level_md2 = val;
   
-  if (Fiid_obj_get (obj_cmd_rs, "admin_level.md5", &val) < 0)
-    goto cleanup;
+  _FIID_OBJ_GET (obj_cmd_rs, "admin_level.md5", &val);
   al->admin_level_md5 = val;
   
-  if (Fiid_obj_get (obj_cmd_rs, "admin_level.straight_password", &val) < 0)
-    goto cleanup;
+  _FIID_OBJ_GET (obj_cmd_rs, "admin_level.straight_password", &val);
   al->admin_level_straight_password = val;
   
-  if (Fiid_obj_get (obj_cmd_rs, "admin_level.oem_proprietary", &val) < 0)
-    goto cleanup;
+  _FIID_OBJ_GET (obj_cmd_rs, "admin_level.oem_proprietary", &val);
   al->admin_level_oem_proprietary = val;
   
-  if (Fiid_obj_get (obj_cmd_rs, "oem_level.none", &val) < 0)
-    goto cleanup;
+  _FIID_OBJ_GET (obj_cmd_rs, "oem_level.none", &val);
   al->oem_level_none = val;
   
-  if (Fiid_obj_get (obj_cmd_rs, "oem_level.md2", &val) < 0)
-    goto cleanup;
+  _FIID_OBJ_GET (obj_cmd_rs, "oem_level.md2", &val);
   al->oem_level_md2 = val;
   
-  if (Fiid_obj_get (obj_cmd_rs, "oem_level.md5", &val) < 0)
-    goto cleanup;
+  _FIID_OBJ_GET (obj_cmd_rs, "oem_level.md5", &val);
   al->oem_level_md5 = val;
   
-  if (Fiid_obj_get (obj_cmd_rs, "oem_level.straight_password", &val) < 0)
-    goto cleanup;
+  _FIID_OBJ_GET (obj_cmd_rs, "oem_level.straight_password", &val);
   al->oem_level_straight_password = val;
   
-  if (Fiid_obj_get (obj_cmd_rs, "oem_level.oem_proprietary", &val) < 0)
-    goto cleanup;
+  _FIID_OBJ_GET (obj_cmd_rs, "oem_level.oem_proprietary", &val);
   al->oem_level_oem_proprietary = val;
   
   rv = CONFIG_ERR_SUCCESS;
  cleanup:
-  Fiid_obj_destroy(obj_cmd_rs);
+  _FIID_OBJ_DESTROY(obj_cmd_rs);
   return (rv);
 }
 
@@ -197,8 +253,7 @@ _set_authentication_type_enables (bmc_config_state_data_t *state_data,
   assert(state_data);
   assert(al);
 
-  if (!(obj_cmd_rs = Fiid_obj_create(tmpl_cmd_set_lan_configuration_parameters_rs)))
-    goto cleanup;
+  _FIID_OBJ_CREATE(obj_cmd_rs, tmpl_cmd_set_lan_configuration_parameters_rs);
 
   if ((ret = get_lan_channel_number (state_data, &channel_number)) != CONFIG_ERR_SUCCESS)
     {
@@ -206,7 +261,7 @@ _set_authentication_type_enables (bmc_config_state_data_t *state_data,
       goto cleanup;
     }
 
-  if (ipmi_cmd_set_lan_configuration_parameters_authentication_type_enables (state_data->dev,
+  if (ipmi_cmd_set_lan_configuration_parameters_authentication_type_enables (state_data->ipmi_ctx,
                                                                              channel_number, 
                                                                              al->callback_level_none,
                                                                              al->callback_level_md2,
@@ -235,25 +290,29 @@ _set_authentication_type_enables (bmc_config_state_data_t *state_data,
                                                                              al->oem_level_oem_proprietary,
                                                                              obj_cmd_rs) < 0)
     {
-      if (state_data->prog_data->args->common.flags & IPMI_FLAGS_DEBUG_DUMP)
-        fprintf(stderr,
-                "ipmi_cmd_set_lan_configuration_parameters_authentication_type_enables: %s\n",
-                ipmi_device_strerror(ipmi_device_errnum(state_data->dev)));
-      rv = CONFIG_ERR_NON_FATAL_ERROR;
+      if (state_data->prog_data->args->config_args.common.debug)
+        pstdout_fprintf(state_data->pstate,
+                        stderr,
+                        "ipmi_cmd_set_lan_configuration_parameters_authentication_type_enables: %s\n",
+                        ipmi_ctx_strerror(ipmi_ctx_errnum(state_data->ipmi_ctx)));
+      if (!IPMI_CTX_ERRNUM_IS_FATAL_ERROR(state_data->ipmi_ctx))
+        rv = CONFIG_ERR_NON_FATAL_ERROR;
       goto cleanup;
     }
   
   rv = CONFIG_ERR_SUCCESS;
  cleanup:
-  Fiid_obj_destroy(obj_cmd_rs);
+  _FIID_OBJ_DESTROY(obj_cmd_rs);
   return (rv);
 }
 
 static uint8_t *
-_authentication_level_ptr (const char *section_name,
+_authentication_level_ptr (bmc_config_state_data_t *state_data,
+                           const char *section_name,
                            const char *key_name,
                            struct bmc_authentication_level *al)
 {
+  assert(state_data);
   assert(key_name);
   assert(al);
 
@@ -308,13 +367,59 @@ _authentication_level_ptr (const char *section_name,
   else if (!strcasecmp(key_name, "OEM_Enable_Auth_Type_OEM_Proprietary"))
     return &al->oem_level_oem_proprietary;
 
-  fprintf(stderr, 
-          "Unknown key '%s' in section '%s'\n", 
-          key_name,
-          section_name);
+  pstdout_fprintf(state_data->pstate,
+                  stderr, 
+                  "Unknown key '%s' in section '%s'\n", 
+                  key_name,
+                  section_name);
   return NULL;
 }
 
+/* based on support flags, determine if checkout is available 
+ * - if we cannot determine support, we always checkout
+ */
+static config_err_t
+_authentication_type_enable_available (bmc_config_state_data_t *state_data,
+                                       const char *section_name,
+                                       const char *key_name,
+                                       unsigned int *available)
+{
+  config_err_t ret;
+
+  assert(state_data);
+  assert(key_name);
+  assert(available);
+
+  /* default to always allow checkout */
+  *available = 1;
+          
+  /* always output under verbose mode */
+  if (state_data->prog_data->args->config_args.verbose)
+    return CONFIG_ERR_SUCCESS;
+
+  if (!state_data->authentication_type_initialized)
+    {
+      if ((ret = _get_authentication_type_support (state_data)) != CONFIG_ERR_SUCCESS)
+        return ret;
+    }
+
+  if (state_data->authentication_type_initialized)
+    {
+      if (stristr(key_name, "None") && !state_data->authentication_type_none)
+        *available = 0;
+      else if (stristr(key_name, "MD2") && !state_data->authentication_type_md2)
+        *available = 0;
+      else if (stristr(key_name, "MD5") && !state_data->authentication_type_md5)
+        *available = 0;
+      else if (stristr(key_name, "Straight_Password") && !state_data->authentication_type_straight_password)
+        *available = 0;
+      else if (stristr(key_name, "OEM_Proprietary") && !state_data->authentication_type_oem_proprietary)
+        *available = 0;
+    }
+
+  return CONFIG_ERR_SUCCESS;
+}
+
 static config_err_t
 _authentication_level_checkout (const char *section_name,
                                 struct config_keyvalue *kv,
@@ -323,19 +428,38 @@ _authentication_level_checkout (const char *section_name,
   bmc_config_state_data_t *state_data = (bmc_config_state_data_t *)arg;
   struct bmc_authentication_level al;
   config_err_t ret;
-  uint8_t *flag;
+  unsigned int available_flag = 1; /* default is to always allow checkout */
+  uint8_t *al_ptr;
 
   if ((ret = _get_authentication_type_enables (state_data, 
                                                &al)) != CONFIG_ERR_SUCCESS)
     return ret;
 
-  if (!(flag = _authentication_level_ptr(section_name, kv->key->key_name, &al)))
-    return CONFIG_ERR_FATAL_ERROR;
+  /* non-fatal error is ok here */
+  ret = _authentication_type_enable_available (state_data,
+                                               section_name,
+                                               kv->key->key_name,
+                                               &available_flag);
+  if (ret == CONFIG_ERR_FATAL_ERROR)
+    return ret;
+      
+  if (available_flag) 
+    {
+      if (!(al_ptr = _authentication_level_ptr(state_data,
+                                               section_name, 
+                                               kv->key->key_name, 
+                                               &al)))
+        return CONFIG_ERR_FATAL_ERROR;
+      
+      if (config_section_update_keyvalue_output(state_data->pstate,
+                                                kv,
+                                                *al_ptr ? "Yes" : "No") < 0)
+        return CONFIG_ERR_FATAL_ERROR;
 
-  if (config_section_update_keyvalue_output(kv, *flag ? "Yes" : "No") < 0)
-    return CONFIG_ERR_FATAL_ERROR;
+      return CONFIG_ERR_SUCCESS;
+    }
   
-  return CONFIG_ERR_SUCCESS;
+  return CONFIG_ERR_NON_FATAL_ERROR;
 }
 
 static config_err_t
@@ -352,7 +476,10 @@ _authentication_level_commit (const char *section_name,
                                                &al)) != CONFIG_ERR_SUCCESS)
     return ret;
 
-  if (!(flag = _authentication_level_ptr(section_name, kv->key->key_name, &al)))
+  if (!(flag = _authentication_level_ptr(state_data,
+                                         section_name, 
+                                         kv->key->key_name, 
+                                         &al)))
     return CONFIG_ERR_FATAL_ERROR;
   
   *flag = same (kv->value_input, "yes");
@@ -376,13 +503,17 @@ bmc_config_lan_conf_auth_section_get (bmc_config_state_data_t *state_data)
     "will also want to configure some of the \"None\" fields to \"Yes\" "
     "to allow \"None\" authentication to work.";
 
-  if (!(lan_conf_auth_section = config_section_create("Lan_Conf_Auth",
+  if (!(lan_conf_auth_section = config_section_create(state_data->pstate,
+                                                      "Lan_Conf_Auth",
                                                       "Lan_Conf_Auth",
                                                       section_comment,
-                                                      0)))
+                                                      0,
+                                                      NULL,
+                                                      NULL)))
     goto cleanup;
 
-  if (config_section_add_key (lan_conf_auth_section,
+  if (config_section_add_key (state_data->pstate,
+                              lan_conf_auth_section,
                               "Callback_Enable_Auth_Type_None",
                               "Possible values: Yes/No",
                               0,
@@ -391,7 +522,8 @@ bmc_config_lan_conf_auth_section_get (bmc_config_state_data_t *state_data)
                               config_yes_no_validate) < 0)
     goto cleanup;
 
-  if (config_section_add_key (lan_conf_auth_section,
+  if (config_section_add_key (state_data->pstate,
+                              lan_conf_auth_section,
                               "Callback_Enable_Auth_Type_MD2",
                               "Possible values: Yes/No",
                               0,
@@ -400,7 +532,8 @@ bmc_config_lan_conf_auth_section_get (bmc_config_state_data_t *state_data)
                               config_yes_no_validate) < 0)
     goto cleanup;
 
-  if (config_section_add_key (lan_conf_auth_section,
+  if (config_section_add_key (state_data->pstate,
+                              lan_conf_auth_section,
                               "Callback_Enable_Auth_Type_MD5",
                               "Possible values: Yes/No",
                               0,
@@ -409,7 +542,8 @@ bmc_config_lan_conf_auth_section_get (bmc_config_state_data_t *state_data)
                               config_yes_no_validate) < 0)
     goto cleanup;
 
-  if (config_section_add_key (lan_conf_auth_section,
+  if (config_section_add_key (state_data->pstate,
+                              lan_conf_auth_section,
                               "Callback_Enable_Auth_Type_Straight_Password",
                               "Possible values: Yes/No",
                               0,
@@ -418,7 +552,8 @@ bmc_config_lan_conf_auth_section_get (bmc_config_state_data_t *state_data)
                               config_yes_no_validate) < 0)
     goto cleanup;
 
-  if (config_section_add_key (lan_conf_auth_section,
+  if (config_section_add_key (state_data->pstate,
+                              lan_conf_auth_section,
                               "Callback_Enable_Auth_Type_OEM_Proprietary",
                               "Possible values: Yes/No",
                               0,
@@ -427,7 +562,8 @@ bmc_config_lan_conf_auth_section_get (bmc_config_state_data_t *state_data)
                               config_yes_no_validate) < 0)
     goto cleanup;
 
-  if (config_section_add_key (lan_conf_auth_section,
+  if (config_section_add_key (state_data->pstate,
+                              lan_conf_auth_section,
                               "User_Enable_Auth_Type_None",
                               "Possible values: Yes/No",
                               0,
@@ -436,7 +572,8 @@ bmc_config_lan_conf_auth_section_get (bmc_config_state_data_t *state_data)
                               config_yes_no_validate) < 0)
     goto cleanup;
 
-  if (config_section_add_key (lan_conf_auth_section,
+  if (config_section_add_key (state_data->pstate,
+                              lan_conf_auth_section,
                               "User_Enable_Auth_Type_MD2",
                               "Possible values: Yes/No",
                               0,
@@ -445,7 +582,8 @@ bmc_config_lan_conf_auth_section_get (bmc_config_state_data_t *state_data)
                               config_yes_no_validate) < 0)
     goto cleanup;
 
-  if (config_section_add_key (lan_conf_auth_section,
+  if (config_section_add_key (state_data->pstate,
+                              lan_conf_auth_section,
                               "User_Enable_Auth_Type_MD5",
                               "Possible values: Yes/No",
                               0,
@@ -454,7 +592,8 @@ bmc_config_lan_conf_auth_section_get (bmc_config_state_data_t *state_data)
                               config_yes_no_validate) < 0)
     goto cleanup;
 
-  if (config_section_add_key (lan_conf_auth_section,
+  if (config_section_add_key (state_data->pstate,
+                              lan_conf_auth_section,
                               "User_Enable_Auth_Type_Straight_Password",
                               "Possible values: Yes/No",
                               0,
@@ -463,7 +602,8 @@ bmc_config_lan_conf_auth_section_get (bmc_config_state_data_t *state_data)
                               config_yes_no_validate) < 0)
     goto cleanup;
 
-  if (config_section_add_key (lan_conf_auth_section,
+  if (config_section_add_key (state_data->pstate,
+                              lan_conf_auth_section,
                               "User_Enable_Auth_Type_OEM_Proprietary",
                               "Possible values: Yes/No",
                               0,
@@ -472,7 +612,8 @@ bmc_config_lan_conf_auth_section_get (bmc_config_state_data_t *state_data)
                               config_yes_no_validate) < 0)
     goto cleanup;
 
-  if (config_section_add_key (lan_conf_auth_section,
+  if (config_section_add_key (state_data->pstate,
+                              lan_conf_auth_section,
                               "Operator_Enable_Auth_Type_None",
                               "Possible values: Yes/No",
                               0,
@@ -481,7 +622,8 @@ bmc_config_lan_conf_auth_section_get (bmc_config_state_data_t *state_data)
                               config_yes_no_validate) < 0)
     goto cleanup;
 
-  if (config_section_add_key (lan_conf_auth_section,
+  if (config_section_add_key (state_data->pstate,
+                              lan_conf_auth_section,
                               "Operator_Enable_Auth_Type_MD2",
                               "Possible values: Yes/No",
                               0,
@@ -490,7 +632,8 @@ bmc_config_lan_conf_auth_section_get (bmc_config_state_data_t *state_data)
                               config_yes_no_validate) < 0)
     goto cleanup;
 
-  if (config_section_add_key (lan_conf_auth_section,
+  if (config_section_add_key (state_data->pstate,
+                              lan_conf_auth_section,
                               "Operator_Enable_Auth_Type_MD5",
                               "Possible values: Yes/No",
                               0,
@@ -499,7 +642,8 @@ bmc_config_lan_conf_auth_section_get (bmc_config_state_data_t *state_data)
                               config_yes_no_validate) < 0)
     goto cleanup;
 
-  if (config_section_add_key (lan_conf_auth_section,
+  if (config_section_add_key (state_data->pstate,
+                              lan_conf_auth_section,
                               "Operator_Enable_Auth_Type_Straight_Password",
                               "Possible values: Yes/No",
                               0,
@@ -508,7 +652,8 @@ bmc_config_lan_conf_auth_section_get (bmc_config_state_data_t *state_data)
                               config_yes_no_validate) < 0)
     goto cleanup;
 
-  if (config_section_add_key (lan_conf_auth_section,
+  if (config_section_add_key (state_data->pstate,
+                              lan_conf_auth_section,
                               "Operator_Enable_Auth_Type_OEM_Proprietary",
                               "Possible values: Yes/No",
                               0,
@@ -517,7 +662,8 @@ bmc_config_lan_conf_auth_section_get (bmc_config_state_data_t *state_data)
                               config_yes_no_validate) < 0)
     goto cleanup;
 
-  if (config_section_add_key (lan_conf_auth_section,
+  if (config_section_add_key (state_data->pstate,
+                              lan_conf_auth_section,
                               "Admin_Enable_Auth_Type_None",
                               "Possible values: Yes/No",
                               0,
@@ -526,7 +672,8 @@ bmc_config_lan_conf_auth_section_get (bmc_config_state_data_t *state_data)
                               config_yes_no_validate) < 0)
     goto cleanup;
 
-  if (config_section_add_key (lan_conf_auth_section,
+  if (config_section_add_key (state_data->pstate,
+                              lan_conf_auth_section,
                               "Admin_Enable_Auth_Type_MD2",
                               "Possible values: Yes/No",
                               0,
@@ -535,7 +682,8 @@ bmc_config_lan_conf_auth_section_get (bmc_config_state_data_t *state_data)
                               config_yes_no_validate) < 0)
     goto cleanup;
 
-  if (config_section_add_key (lan_conf_auth_section,
+  if (config_section_add_key (state_data->pstate,
+                              lan_conf_auth_section,
                               "Admin_Enable_Auth_Type_MD5",
                               "Possible values: Yes/No",
                               0,
@@ -544,7 +692,8 @@ bmc_config_lan_conf_auth_section_get (bmc_config_state_data_t *state_data)
                               config_yes_no_validate) < 0)
     goto cleanup;
 
-  if (config_section_add_key (lan_conf_auth_section,
+  if (config_section_add_key (state_data->pstate,
+                              lan_conf_auth_section,
                               "Admin_Enable_Auth_Type_Straight_Password",
                               "Possible values: Yes/No",
                               0,
@@ -553,7 +702,8 @@ bmc_config_lan_conf_auth_section_get (bmc_config_state_data_t *state_data)
                               config_yes_no_validate) < 0)
     goto cleanup;
 
-  if (config_section_add_key (lan_conf_auth_section,
+  if (config_section_add_key (state_data->pstate,
+                              lan_conf_auth_section,
                               "Admin_Enable_Auth_Type_OEM_Proprietary",
                               "Possible values: Yes/No",
                               0,
@@ -562,7 +712,8 @@ bmc_config_lan_conf_auth_section_get (bmc_config_state_data_t *state_data)
                               config_yes_no_validate) < 0)
     goto cleanup;
 
-  if (config_section_add_key (lan_conf_auth_section,
+  if (config_section_add_key (state_data->pstate,
+                              lan_conf_auth_section,
                               "OEM_Enable_Auth_Type_None",
                               "Possible values: Yes/No",
                               0,
@@ -571,7 +722,8 @@ bmc_config_lan_conf_auth_section_get (bmc_config_state_data_t *state_data)
                               config_yes_no_validate) < 0)
     goto cleanup;
 
-  if (config_section_add_key (lan_conf_auth_section,
+  if (config_section_add_key (state_data->pstate,
+                              lan_conf_auth_section,
                               "OEM_Enable_Auth_Type_MD2",
                               "Possible values: Yes/No",
                               0,
@@ -580,7 +732,8 @@ bmc_config_lan_conf_auth_section_get (bmc_config_state_data_t *state_data)
                               config_yes_no_validate) < 0)
     goto cleanup;
 
-  if (config_section_add_key (lan_conf_auth_section,
+  if (config_section_add_key (state_data->pstate,
+                              lan_conf_auth_section,
                               "OEM_Enable_Auth_Type_MD5",
                               "Possible values: Yes/No",
                               0,
@@ -589,7 +742,8 @@ bmc_config_lan_conf_auth_section_get (bmc_config_state_data_t *state_data)
                               config_yes_no_validate) < 0)
     goto cleanup;
 
-  if (config_section_add_key (lan_conf_auth_section,
+  if (config_section_add_key (state_data->pstate,
+                              lan_conf_auth_section,
                               "OEM_Enable_Auth_Type_Straight_Password",
                               "Possible values: Yes/No",
                               0,
@@ -598,7 +752,8 @@ bmc_config_lan_conf_auth_section_get (bmc_config_state_data_t *state_data)
                               config_yes_no_validate) < 0)
     goto cleanup;
 
-  if (config_section_add_key (lan_conf_auth_section,
+  if (config_section_add_key (state_data->pstate,
+                              lan_conf_auth_section,
                               "OEM_Enable_Auth_Type_OEM_Proprietary",
                               "Possible values: Yes/No",
                               0,
@@ -611,6 +766,6 @@ bmc_config_lan_conf_auth_section_get (bmc_config_state_data_t *state_data)
 
  cleanup:
   if (lan_conf_auth_section)
-    config_section_destroy(lan_conf_auth_section);
+    config_section_destroy(state_data->pstate, lan_conf_auth_section);
   return NULL;
 }
