@@ -1,7 +1,5 @@
 /* 
-   ipmi-chassis.h - IPMI Chassis information
-   
-   Copyright (C) 2007 FreeIPMI Core Team
+   Copyright (C) 2007-2008 FreeIPMI Core Team
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -23,21 +21,45 @@
 
 #include <freeipmi/freeipmi.h>
 
-#include "cmdline-parse-common.h"
+#include "tool-cmdline-common.h"
 #include "pstdout.h"
 
-enum ipmi_chassis_keys
+enum ipmi_chassis_argp_option_keys
   {
-    IPMI_CHASSIS_KEY_BOOT_TYPE = 150,
-    IPMI_CHASSIS_KEY_LOCK_OUT_RESET_BUTTON,
-    IPMI_CHASSIS_KEY_SCREEN_BLANK,
-    IPMI_CHASSIS_KEY_BOOT_DEVICE_SELECTOR,
-    IPMI_CHASSIS_KEY_LOCK_KEYBOARD,
-    IPMI_CHASSIS_KEY_CLEAR_CMOS,
-    IPMI_CHASSIS_KEY_CONSOLE_REDIRECTION,
-    IPMI_CHASSIS_KEY_USER_PASSWORD_BYPASS,
-    IPMI_CHASSIS_KEY_FORCE_PROGRESS_EVENT_TRAPS,
-    IPMI_CHASSIS_KEY_FIRMWARE_BIOS_VERBOSITY,
+    GET_CAPABILITIES_KEY = 'c',
+    GET_STATUS_KEY = 's',
+    CHASSIS_CONTROL_KEY = 'O',
+    CHASSIS_IDENTIFY_KEY = 'i',
+    GET_SYSTEM_RESTART_CAUSE_KEY = 'R',
+    GET_POWER_ON_HOURS_COUNTER_KEY = 'H',
+    SET_POWER_CYCLE_INTERVAL_KEY = 'S',
+    GET_BOOT_FLAGS_KEY = 'G',
+    SET_BOOT_FLAGS_KEY = 'L',
+    SET_BOOT_FLAGS_BOOT_TYPE_KEY = 160,
+    SET_BOOT_FLAGS_LOCK_OUT_RESET_BUTTON_KEY = 161,
+    SET_BOOT_FLAGS_SCREEN_BLANK_KEY = 162,
+    SET_BOOT_FLAGS_BOOT_DEVICE_KEY = 163,
+    SET_BOOT_FLAGS_LOCK_KEYBOARD_KEY = 164,
+    SET_BOOT_FLAGS_CMOS_CLEAR_KEY = 165,
+    SET_BOOT_FLAGS_CONSOLE_REDIRECTION_KEY = 166,
+    SET_BOOT_FLAGS_USER_PASSWORD_BYPASS_KEY = 167,
+    SET_BOOT_FLAGS_FORCE_PROGRESS_EVENT_TRAPS_KEY = 168,
+    SET_BOOT_FLAGS_FIRMWARE_BIOS_VERBOSITY_KEY = 169,
+    SET_POWER_RESTORE_POLICY_KEY = 'X',
+  };
+
+enum ipmi_chassis_cmds
+  {
+    CHASSIS_CMD_GET_CHASSIS_CAPABILITIES = 0,
+    CHASSIS_CMD_GET_CHASSIS_STATUS = 1,
+    CHASSIS_CMD_CHASSIS_CONTROL = 2,
+    CHASSIS_CMD_CHASSIS_IDENTIFY = 3,
+    CHASSIS_CMD_SET_POWER_RESTORE_POLICY = 4, 
+    CHASSIS_CMD_SET_POWER_CYCLE_INTERVAL = 5,
+    CHASSIS_CMD_GET_SYSTEM_RESTART_CAUSE = 6,
+    CHASSIS_CMD_GET_POWER_ON_HOURS_COUNTER = 7,
+    CHASSIS_CMD_SET_SYSTEM_BOOT_OPTIONS = 8,
+    CHASSIS_CMD_GET_SYSTEM_BOOT_OPTIONS = 9,
   };
 
 struct cmd_boot_option
@@ -45,9 +67,9 @@ struct cmd_boot_option
   int8_t bios_boot_type;
   int8_t lock_out_reset_button;
   int8_t screen_blank;
-  int8_t boot_device_selector;
+  int8_t boot_device;
   int8_t lock_keyboard;
-  int8_t clear_cmos;
+  int8_t cmos_clear;
   int8_t console_redirection;
   int8_t user_password_bypass;
   int8_t force_progress_event_traps;
@@ -87,7 +109,7 @@ typedef struct ipmi_chassis_prog_data
 typedef struct ipmi_chassis_state_data
 {
   ipmi_chassis_prog_data_t *prog_data;
-  ipmi_device_t dev;
+  ipmi_ctx_t ipmi_ctx;
   pstdout_state_t pstate;
 } ipmi_chassis_state_data_t;
 
