@@ -1,9 +1,5 @@
 /* 
-   $Id: ipmi-raw-argp.c,v 1.22 2007/09/05 20:13:26 chu11 Exp $ 
-   
-   ipmi-raw-argp.c - ipmi-raw command line argument parser.
-   
-   Copyright (C) 2005 FreeIPMI Core Team
+   Copyright (C) 2005-2008 FreeIPMI Core Team
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -22,36 +18,47 @@
 
 #if HAVE_CONFIG_H
 #include "config.h"
-#endif
+#endif /* HAVE_CONFIG_H */
 
 #include <stdio.h>
 #include <stdlib.h>
 #if STDC_HEADERS
 #include <string.h>
 #endif /* STDC_HEADERS */
+#if HAVE_UNISTD_H
+#include <unistd.h>
+#endif /* HAVE_UNISTD_H */
+#if HAVE_ARGP_H
 #include <argp.h>
+#else /* !HAVE_ARGP_H */
+#include "freeipmi-argp.h"
+#endif /* !HAVE_ARGP_H */
+#include <limits.h>
+#include <errno.h>
 
-#include "cmdline-parse-common.h"
 #include "ipmi-raw.h"
 #include "ipmi-raw-argp.h"
 
 #include "freeipmi-portability.h"
-
-static error_t parse_opt (int key, char *arg, struct argp_state *state);
+#include "tool-cmdline-common.h"
+#include "tool-config-file-common.h"
 
 const char *argp_program_version = 
-"FreeIPMI Raw [ipmi-raw-" PACKAGE_VERSION "]\n"
-"Copyright (C) 2003-2005 FreeIPMI Core Team\n"
-"This program is free software; you may redistribute it under the terms of\n"
-"the GNU General Public License.  This program has absolutely no warranty.";
+  "ipmi-raw - " PACKAGE_VERSION "\n"
+  "Copyright (C) 2005-2008 FreeIPMI Core Team\n"
+  "This program is free software; you may redistribute it under the terms of\n"
+  "the GNU General Public License.  This program has absolutely no warranty.";
 
-const char *argp_program_bug_address = "<freeipmi-devel@gnu.org>";
+const char *argp_program_bug_address = 
+  "<" PACKAGE_BUGREPORT ">";
 
-static char doc[] = "IPMI Raw - executes IPMI commands by hex values.";
+static char cmdline_doc[] = 
+  "ipmi-raw - execute IPMI commands by hex values";
 
-static char args_doc[] = "[COMMAND-HEX-BYTES]";
+static char cmdline_args_doc[] = 
+  "[<lun> <netfn> COMMAND-HEX-BYTES]";
 
-static struct argp_option options[] = 
+static struct argp_option cmdline_options[] = 
   {
     ARGP_COMMON_OPTIONS_DRIVER,
     ARGP_COMMON_OPTIONS_INBAND,
@@ -59,18 +66,29 @@ static struct argp_option options[] =
     ARGP_COMMON_OPTIONS_AUTHENTICATION_TYPE,
     ARGP_COMMON_OPTIONS_CIPHER_SUITE_ID,
     ARGP_COMMON_OPTIONS_PRIVILEGE_LEVEL_USER,
+    ARGP_COMMON_OPTIONS_CONFIG_FILE,
     ARGP_COMMON_OPTIONS_WORKAROUND_FLAGS,
     ARGP_COMMON_HOSTRANGED_OPTIONS,
     ARGP_COMMON_OPTIONS_DEBUG,
     {"file", CMD_FILE_KEY, "CMD-FILE", 0, 
-     "Specify a file to read command requests from.", 25}, 
+     "Specify a file to read command requests from.", 30}, 
     { 0 }
   };
 
-static struct argp argp = { options, parse_opt, args_doc, doc };
+static error_t cmdline_parse (int key, char *arg, struct argp_state *state);
+
+static struct argp cmdline_argp = { cmdline_options,
+                                    cmdline_parse, 
+                                    cmdline_args_doc, 
+                                    cmdline_doc };
+
+static struct argp cmdline_config_file_argp = { cmdline_options,
+                                                cmdline_config_file_parse,
+                                                cmdline_args_doc,
+                                                cmdline_doc };
 
 static error_t 
-parse_opt (int key, char *arg, struct argp_state *state)
+cmdline_parse (int key, char *arg, struct argp_state *state)
 {
   struct ipmi_raw_arguments *cmd_args = state->input;
   error_t ret;
@@ -78,7 +96,11 @@ parse_opt (int key, char *arg, struct argp_state *state)
   switch (key)
     {
     case CMD_FILE_KEY:
-      cmd_args->cmd_file = strdup (arg);
+      if (!(cmd_args->cmd_file = strdup (arg)))
+        {
+          perror("strdup");
+          exit(1);
+        }
       break;
     case ARGP_KEY_ARG:
       {
@@ -94,8 +116,7 @@ parse_opt (int key, char *arg, struct argp_state *state)
         if (*arg == '\0')
           {
             fprintf (stderr, "invalid hex byte argument\n");
-            argp_usage (state);
-            return (-1);
+            exit(1);
           }
 
 	for (i = 0; arg[i] != (char) NULL; i++)
@@ -103,15 +124,13 @@ parse_opt (int key, char *arg, struct argp_state *state)
 	    if (i >= 2)
 	      {
 		fprintf (stderr, "invalid hex byte argument\n");
-		argp_usage (state);
-		return (-1);
+                exit(1);
 	      }
 	    
 	    if (isxdigit (arg[i]) == 0)
 	      {
 		fprintf (stderr, "invalid hex byte argument\n");
-		argp_usage (state);
-		return (-1);
+                exit(1);
 	      }
 	  }
 	
@@ -132,17 +151,54 @@ parse_opt (int key, char *arg, struct argp_state *state)
   return 0;
 }
 
+static void
+_ipmi_raw_config_file_parse(struct ipmi_raw_arguments *cmd_args)
+{
+  if (config_file_parse (cmd_args->common.config_file,
+                         0,
+                         &(cmd_args->common),
+                         NULL,
+                         &(cmd_args->hostrange),
+                         CONFIG_FILE_INBAND | CONFIG_FILE_OUTOFBAND | CONFIG_FILE_HOSTRANGE,
+                         CONFIG_FILE_TOOL_IPMI_RAW,
+                         NULL) < 0)
+    {
+      fprintf(stderr, "config_file_parse: %s\n", strerror(errno));
+      exit(1);
+    }
+}
+
 void 
 ipmi_raw_argp_parse (int argc, char **argv, struct ipmi_raw_arguments *cmd_args)
 {
-  init_common_cmd_args (&(cmd_args->common));
+  long arg_max;
+
+  init_common_cmd_args_user (&(cmd_args->common));
   init_hostrange_cmd_args (&(cmd_args->hostrange));
 
   cmd_args->cmd_file = NULL;
-  memset (cmd_args->cmd, 0, sizeof(cmd_args->cmd));
+  errno = 0;
+  if ((arg_max = sysconf(_SC_ARG_MAX)) < 0)
+    {
+      if (errno)
+        {
+          perror("sysconf");
+          exit(1);
+        }
+      arg_max = LONG_MAX;
+    }
+  if (!(cmd_args->cmd = calloc(arg_max, sizeof(uint8_t))))
+    {
+      perror("calloc");
+      exit(1);
+    }
   cmd_args->cmd_length = 0;
 
-  argp_parse (&argp, argc, argv, ARGP_IN_ORDER, NULL, cmd_args);
+  argp_parse (&cmdline_config_file_argp, argc, argv, ARGP_IN_ORDER, NULL, &(cmd_args->common));
+
+  _ipmi_raw_config_file_parse(cmd_args);
+
+  argp_parse (&cmdline_argp, argc, argv, ARGP_IN_ORDER, NULL, cmd_args);
   verify_common_cmd_args (&(cmd_args->common));
   verify_hostrange_cmd_args (&(cmd_args->hostrange));
 }
