@@ -1,45 +1,47 @@
 /*
-ipmi-raw.c: executes IPMI commands by hex values.
-Copyright (C) 2005 FreeIPMI Core Team
+  Copyright (C) 2005-2008 FreeIPMI Core Team
 
-This program is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2, or (at your option)
-any later version.
-
-This program is distributed in the hope that it will be useful, but
-WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with this program; if not, write to the Free Software
-Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA
+  This program is free software; you can redistribute it and/or modify
+  it under the terms of the GNU General Public License as published by
+  the Free Software Foundation; either version 2, or (at your option)
+  any later version.
+  
+  This program is distributed in the hope that it will be useful, but
+  WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+  General Public License for more details.
+  
+  You should have received a copy of the GNU General Public License
+  along with this program; if not, write to the Free Software
+  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA
 */
 
 #if HAVE_CONFIG_H
 #include "config.h"
-#endif
+#endif /* HAVE_CONFIG_H */
 
 #include <stdio.h>
 #include <stdlib.h>
 #if STDC_HEADERS
 #include <string.h>
 #endif /* STDC_HEADERS */
-#include <argp.h>
+#if HAVE_UNISTD_H
+#include <unistd.h>
+#endif /* HAVE_UNISTD_H */
 #include <limits.h>
+#include <errno.h>
 #include <assert.h>
 
 #include <freeipmi/freeipmi.h>
-#include <freeipmi/udm/udm.h>
 
-#include "cmdline-parse-common.h"
-#include "freeipmi-portability.h"
 #include "ipmi-raw.h"
 #include "ipmi-raw-argp.h"
-#include "tool-common.h"
+
+#include "freeipmi-portability.h"
 #include "pstdout.h"
-#include "hostrange.h"
+#include "tool-common.h"
+#include "tool-cmdline-common.h"
+#include "tool-hostrange-common.h"
 
 int
 ipmi_raw_cmdline (ipmi_raw_state_data_t *state_data)
@@ -48,8 +50,10 @@ ipmi_raw_cmdline (ipmi_raw_state_data_t *state_data)
   uint8_t *bytes_rq = NULL;
   int send_len;
   int i;
-  uint8_t bytes_rs[ARG_MAX];
+  uint8_t *bytes_rs = NULL;
+  long arg_max;
   int32_t rs_len;
+  int rv = -1;
 
   assert(state_data);
   assert(state_data->prog_data->args->cmd);
@@ -65,16 +69,33 @@ ipmi_raw_cmdline (ipmi_raw_state_data_t *state_data)
       pstdout_fprintf(state_data->pstate, 
                       stderr, 
                       "Invalid number of hex bytes\n");
-      return (-1);
+      goto cleanup;
     }
 
-  if ((rs_len = ipmi_cmd_raw (state_data->dev, 
+  errno = 0;
+  if ((arg_max = sysconf(_SC_ARG_MAX)) < 0)
+    {
+      if (errno)
+        {
+          pstdout_perror(state_data->pstate, "sysconf");
+          goto cleanup;
+        }
+      arg_max = LONG_MAX;
+    }
+
+  if (!(bytes_rs = calloc(arg_max, sizeof(uint8_t))))
+    {
+      pstdout_perror(state_data->pstate, "calloc");
+      goto cleanup;
+    }
+
+  if ((rs_len = ipmi_cmd_raw (state_data->ipmi_ctx, 
                               bytes_rq[0],
                               bytes_rq[1],
                               &bytes_rq[2],
                               send_len - 2, 
                               bytes_rs, 
-                              ARG_MAX)) >= 0)
+                              arg_max)) >= 0)
     {
       pstdout_printf (state_data->pstate, "rcvd: ");
       for (i = 0; i < rs_len; i++)
@@ -86,11 +107,15 @@ ipmi_raw_cmdline (ipmi_raw_state_data_t *state_data)
       pstdout_fprintf(state_data->pstate,
                       stderr,
                       "ipmi_cmd_raw: %s\n",
-                      ipmi_device_strerror(ipmi_device_errnum(state_data->dev)));
-      return -1;
+                      ipmi_ctx_strerror(ipmi_ctx_errnum(state_data->ipmi_ctx)));
+      goto cleanup;
     }
 
-  return 0;
+  rv = 0;
+ cleanup:
+  if (bytes_rs)
+    free(bytes_rs);
+  return rv;
 }
 
 static int 
@@ -242,7 +267,7 @@ ipmi_raw_stream (ipmi_raw_state_data_t *state_data, FILE *stream)
           goto end_loop;
         }
 
-      if ((rs_len = ipmi_cmd_raw (state_data->dev, 
+      if ((rs_len = ipmi_cmd_raw (state_data->ipmi_ctx, 
 				  bytes_rq[0], 
 				  bytes_rq[1], 
 				  &bytes_rq[2], 
@@ -253,7 +278,7 @@ ipmi_raw_stream (ipmi_raw_state_data_t *state_data, FILE *stream)
           pstdout_fprintf(state_data->pstate,
                           stderr,
                           "ipmi_cmd_raw: %s\n",
-                          ipmi_device_strerror(ipmi_device_errnum(state_data->dev)));
+                          ipmi_ctx_strerror(ipmi_ctx_errnum(state_data->ipmi_ctx)));
           goto end_loop;
         }
       
@@ -297,10 +322,11 @@ run_cmd_args (ipmi_raw_state_data_t *state_data)
 
   args = state_data->prog_data->args;
 
-  if (args->cmd && args->cmd_length)
+  if (args->cmd_length)
     {
       if (ipmi_raw_cmdline(state_data) < 0)
         goto cleanup;
+      return 0;
     }
 
   if (args->cmd_file)
@@ -312,17 +338,11 @@ run_cmd_args (ipmi_raw_state_data_t *state_data)
         }
     }
   else 
-    {
-      if (args->cmd_length == 0)
-	infile = stdin;
-    }
-
-  if (infile)
-    {
-      if (ipmi_raw_stream (state_data, infile) < 0)
-        goto cleanup;
-    }
+    infile = stdin;
 
+  if (ipmi_raw_stream (state_data, infile) < 0)
+    goto cleanup;
+  
   rv = 0;
  cleanup:
   if (infile && infile != stdin)
@@ -337,18 +357,20 @@ _ipmi_raw (pstdout_state_t pstate,
 {
   ipmi_raw_state_data_t state_data;
   ipmi_raw_prog_data_t *prog_data;
-  ipmi_device_t dev = NULL;
-  char errmsg[IPMI_DEVICE_OPEN_ERRMSGLEN];
+  char errmsg[IPMI_OPEN_ERRMSGLEN];
   int exit_code = -1;
 
   prog_data = (ipmi_raw_prog_data_t *)arg;
   memset(&state_data, '\0', sizeof(ipmi_raw_state_data_t));
 
-  if (!(dev = ipmi_device_open(prog_data->progname,
-                               hostname,
-                               &(prog_data->args->common),
-                               errmsg,
-                               IPMI_DEVICE_OPEN_ERRMSGLEN)))
+  state_data.prog_data = prog_data;
+  state_data.pstate = pstate;
+
+  if (!(state_data.ipmi_ctx = ipmi_open(prog_data->progname,
+                                        hostname,
+                                        &(prog_data->args->common),
+                                        errmsg,
+                                        IPMI_OPEN_ERRMSGLEN)))
     {
       pstdout_fprintf(pstate,
                       stderr,
@@ -358,10 +380,6 @@ _ipmi_raw (pstdout_state_t pstate,
       goto cleanup;
     }
 
-  state_data.dev = dev;
-  state_data.prog_data = prog_data;
-  state_data.pstate = pstate;
-
   if (run_cmd_args (&state_data) < 0)
     {
       exit_code = EXIT_FAILURE;
@@ -370,10 +388,10 @@ _ipmi_raw (pstdout_state_t pstate,
 
   exit_code = 0;
  cleanup:
-  if (dev)
+  if (state_data.ipmi_ctx)
     {
-      ipmi_close_device (dev);
-      ipmi_device_destroy (dev);
+      ipmi_ctx_close (state_data.ipmi_ctx);
+      ipmi_ctx_destroy (state_data.ipmi_ctx);
     }
   return exit_code;
 }
@@ -388,15 +406,17 @@ main (int argc, char **argv)
 
   ipmi_disable_coredump();
 
+  memset(&prog_data, '\0', sizeof(ipmi_raw_prog_data_t));
   prog_data.progname = argv[0];
   ipmi_raw_argp_parse (argc, argv, &cmd_args);
   prog_data.args = &cmd_args;
 
   if (pstdout_setup(&(prog_data.args->common.hostname),
-                    prog_data.args->hostrange.buffer_hostrange_output,
-                    prog_data.args->hostrange.consolidate_hostrange_output,
+                    prog_data.args->hostrange.buffer_output,
+                    prog_data.args->hostrange.consolidate_output,
                     prog_data.args->hostrange.fanout,
-                    prog_data.args->hostrange.eliminate) < 0)
+                    prog_data.args->hostrange.eliminate,
+                    prog_data.args->hostrange.always_prefix) < 0)
     {
       exit_code = EXIT_FAILURE;
       goto cleanup;
