@@ -1,9 +1,5 @@
 /* 
-   $Id: ipmi-raw.h,v 1.5 2007/09/05 20:13:27 chu11 Exp $ 
-   
-   ipmi-raw.h - ipmi-raw command line argument parser.
-   
-   Copyright (C) 2005 FreeIPMI Core Team
+   Copyright (C) 2005-2008 FreeIPMI Core Team
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -24,9 +20,8 @@
 #define _IPMI_RAW_H
 
 #include <freeipmi/freeipmi.h>
-#include <limits.h>	/* ARG_MAX */
 
-#include "cmdline-parse-common.h"
+#include "tool-cmdline-common.h"
 #include "pstdout.h"
 
 enum ipmi_raw_argp_option_keys
@@ -39,7 +34,7 @@ struct ipmi_raw_arguments
   struct common_cmd_args common;
   struct hostrange_cmd_args hostrange;
   char *cmd_file;
-  uint8_t cmd[ARG_MAX];
+  uint8_t *cmd;
   int cmd_length;
 };
 
@@ -52,7 +47,7 @@ typedef struct ipmi_raw_prog_data
 typedef struct ipmi_raw_state_data
 {
   ipmi_raw_prog_data_t *prog_data;
-  ipmi_device_t dev;
+  ipmi_ctx_t ipmi_ctx;
   pstdout_state_t pstate;
 } ipmi_raw_state_data_t;
 
