@@ -1,7 +1,7 @@
 /*****************************************************************************\
- *  $Id: ipmiconsole_defs.h,v 1.62 2007/10/18 16:18:47 chu11 Exp $
+ *  $Id: ipmiconsole_defs.h,v 1.65.10.2 2008/12/09 18:42:13 chu11 Exp $
  *****************************************************************************
- *  Copyright (C) 2007 Lawrence Livermore National Security, LLC.
+ *  Copyright (C) 2007-2008 Lawrence Livermore National Security, LLC.
  *  Copyright (C) 2006-2007 The Regents of the University of California.
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
  *  Written by Albert Chu <chu11@llnl.gov>
@@ -31,9 +31,9 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <stdint.h>
-#ifdef WITH_PTHREADS
+#ifdef HAVE_PTHREAD_H
 #include <pthread.h>
-#endif /* WITH_PTHREADS */
+#endif /* HAVE_PTHREAD_H */
 #if TIME_WITH_SYS_TIME
 #include <sys/time.h>
 #include <time.h>
@@ -48,7 +48,7 @@
 #include <netinet/in.h>
 #include <freeipmi/freeipmi.h>
 
-#include "cbuf.h"
+#include "scbuf.h"
 
 #ifndef _IPMICONSOLE_DEFS_H
 #define _IPMICONSOLE_DEFS_H
@@ -261,13 +261,13 @@ struct ipmiconsole_ctx_connection {
   /* File Descriptor User Interface */
   int user_fd;                  /* never touched by this library */
   int ipmiconsole_fd;
-  cbuf_t console_remote_console_to_bmc;
-  cbuf_t console_bmc_to_remote_console;
+  scbuf_t console_remote_console_to_bmc;
+  scbuf_t console_bmc_to_remote_console;
 
   /* Connection Data */
   int ipmi_fd;
-  cbuf_t ipmi_from_bmc;
-  cbuf_t ipmi_to_bmc;
+  scbuf_t ipmi_from_bmc;
+  scbuf_t ipmi_to_bmc;
 
   /* Pipe for non-fd communication: from API to engine */
   int asynccomm[2];
@@ -347,6 +347,7 @@ struct ipmiconsole_ctx_session {
   uint32_t retransmission_count;
   uint32_t errors_count;
   uint32_t session_sequence_number_errors_count;
+  uint32_t activate_payloads_count;
   uint32_t deactivate_active_payloads_count;
   uint32_t highest_received_sequence_number;
   /* need to also store bytes read from a previous seq num */
