@@ -1,7 +1,7 @@
 /*****************************************************************************\
- *  $Id: ipmiconsole_packet.c,v 1.15 2007/10/18 16:18:47 chu11 Exp $
+ *  $Id: ipmiconsole_packet.c,v 1.26.4.3 2008/12/09 18:42:13 chu11 Exp $
  *****************************************************************************
- *  Copyright (C) 2007 Lawrence Livermore National Security, LLC.
+ *  Copyright (C) 2007-2008 Lawrence Livermore National Security, LLC.
  *  Copyright (C) 2006-2007 The Regents of the University of California.
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
  *  Written by Albert Chu <chu11@llnl.gov>
@@ -43,13 +43,16 @@
 #include "ipmiconsole.h"
 #include "ipmiconsole_defs.h"
 
-#include "cbuf.h"
-#include "secure.h"
 #include "ipmiconsole_packet.h"
 #include "ipmiconsole_ctx.h"
 #include "ipmiconsole_debug.h"
 #include "ipmiconsole_fiid_wrappers.h"
 
+#include "freeipmi-portability.h"
+#include "cbuf.h"
+#include "debug-util.h"
+#include "secure.h"
+
 #define IPMICONSOLE_MAX_PACKET_DUMP_HDR_LEN 1024
 
 fiid_field_t *
@@ -186,12 +189,9 @@ _packet_dump_hdr(ipmiconsole_ctx_t c,
                  char *hdr,
                  unsigned int hdrlen)
 {
-  char *fmt =
-    "================================================\n"
-    "%s\n"
-    "================================================";
-  char *str;
-  int len;
+  uint8_t packet_type;
+  uint8_t packet_direction;
+  const char *str_cmd = NULL;
 
   assert(c);
   assert(c->magic == IPMICONSOLE_CTX_MAGIC);
@@ -199,76 +199,79 @@ _packet_dump_hdr(ipmiconsole_ctx_t c,
   assert(hdr);
   assert(hdrlen);
 
-  if (p == IPMICONSOLE_PACKET_TYPE_GET_AUTHENTICATION_CAPABILITIES_V20_RQ)
-    str = "= Get Authentication Capabilities V20 Request  =";
-  else if (p == IPMICONSOLE_PACKET_TYPE_GET_AUTHENTICATION_CAPABILITIES_V20_RS)
-    str = "= Get Authentication Capabilities V20 Response =";
-  else if (p == IPMICONSOLE_PACKET_TYPE_OPEN_SESSION_REQUEST)
-    str = "= Open Session Request                         =";
-  else if (p == IPMICONSOLE_PACKET_TYPE_OPEN_SESSION_RESPONSE)
-    str = "= Open Session Response                        =";
+  if (p == IPMICONSOLE_PACKET_TYPE_GET_AUTHENTICATION_CAPABILITIES_V20_RQ
+      || p == IPMICONSOLE_PACKET_TYPE_GET_AUTHENTICATION_CAPABILITIES_V20_RS)
+    packet_type = DEBUG_UTIL_TYPE_IPMI_1_5;
+  else
+    packet_type = DEBUG_UTIL_TYPE_IPMI_2_0;
+
+  if (p == IPMICONSOLE_PACKET_TYPE_GET_AUTHENTICATION_CAPABILITIES_V20_RQ
+      || p == IPMICONSOLE_PACKET_TYPE_GET_AUTHENTICATION_CAPABILITIES_V20_RS)
+    str_cmd = ipmi_cmd_str(IPMI_NET_FN_APP_RQ, IPMI_CMD_GET_CHANNEL_AUTHENTICATION_CAPABILITIES);
+  else if (p == IPMICONSOLE_PACKET_TYPE_OPEN_SESSION_REQUEST
+           || p == IPMICONSOLE_PACKET_TYPE_OPEN_SESSION_RESPONSE)
+    str_cmd = DEBUG_UTIL_OPEN_SESSION_STR;
   else if (p == IPMICONSOLE_PACKET_TYPE_RAKP_MESSAGE_1)
-    str = "= Rakp Message 1 Request                       =";
+    str_cmd = DEBUG_UTIL_RAKP_1_STR;
   else if (p == IPMICONSOLE_PACKET_TYPE_RAKP_MESSAGE_2)
-    str = "= Rakp Message 2 Response                      =";
+    str_cmd = DEBUG_UTIL_RAKP_2_STR;
   else if (p == IPMICONSOLE_PACKET_TYPE_RAKP_MESSAGE_3)
-    str = "= Rakp Message 3 Request                       =";
+    str_cmd = DEBUG_UTIL_RAKP_3_STR;
   else if (p == IPMICONSOLE_PACKET_TYPE_RAKP_MESSAGE_4)
-    str = "= Rakp Message 4 Response                      =";
-  else if (p == IPMICONSOLE_PACKET_TYPE_SET_SESSION_PRIVILEGE_LEVEL_RQ)
-    str = "= Set Session Privilege Level Request          =";
-  else if (p == IPMICONSOLE_PACKET_TYPE_SET_SESSION_PRIVILEGE_LEVEL_RS)
-    str = "= Set Session Privilege Level Response         =";
-  else if (p == IPMICONSOLE_PACKET_TYPE_GET_CHANNEL_PAYLOAD_SUPPORT_RQ)
-    str = "= Get Channel Payload Support Request          =";
-  else if (p == IPMICONSOLE_PACKET_TYPE_GET_CHANNEL_PAYLOAD_SUPPORT_RS)
-    str = "= Get Channel Payload Support Response         =";
-  else if (p == IPMICONSOLE_PACKET_TYPE_GET_PAYLOAD_ACTIVATION_STATUS_RQ)
-    str = "= Get Payload Activation Status Request        =";
-  else if (p == IPMICONSOLE_PACKET_TYPE_GET_PAYLOAD_ACTIVATION_STATUS_RS)
-    str = "= Get Payload Activation Status Response       =";
-  else if (p == IPMICONSOLE_PACKET_TYPE_ACTIVATE_PAYLOAD_RQ)
-    str = "= Activate Payload Request                     =";
-  else if (p == IPMICONSOLE_PACKET_TYPE_ACTIVATE_PAYLOAD_RS)
-    str = "= Activate Payload Response                    =";
+    str_cmd = DEBUG_UTIL_RAKP_4_STR;
+  else if (p == IPMICONSOLE_PACKET_TYPE_SET_SESSION_PRIVILEGE_LEVEL_RQ
+           || p == IPMICONSOLE_PACKET_TYPE_SET_SESSION_PRIVILEGE_LEVEL_RS)
+    str_cmd = ipmi_cmd_str(IPMI_NET_FN_APP_RQ, IPMI_CMD_SET_SESSION_PRIVILEGE_LEVEL);
+  else if (p == IPMICONSOLE_PACKET_TYPE_GET_CHANNEL_PAYLOAD_SUPPORT_RQ
+           || p == IPMICONSOLE_PACKET_TYPE_GET_CHANNEL_PAYLOAD_SUPPORT_RS)
+    str_cmd = ipmi_cmd_str(IPMI_NET_FN_APP_RQ, IPMI_CMD_GET_CHANNEL_PAYLOAD_SUPPORT);
+  else if (p == IPMICONSOLE_PACKET_TYPE_GET_PAYLOAD_ACTIVATION_STATUS_RQ
+           || p == IPMICONSOLE_PACKET_TYPE_GET_PAYLOAD_ACTIVATION_STATUS_RS)
+    str_cmd = ipmi_cmd_str(IPMI_NET_FN_APP_RQ, IPMI_CMD_GET_PAYLOAD_ACTIVATION_STATUS);
+  else if (p == IPMICONSOLE_PACKET_TYPE_ACTIVATE_PAYLOAD_RQ
+           || p == IPMICONSOLE_PACKET_TYPE_ACTIVATE_PAYLOAD_RS)
+    str_cmd = ipmi_cmd_str(IPMI_NET_FN_APP_RQ, IPMI_CMD_ACTIVATE_PAYLOAD);
   else if (p == IPMICONSOLE_PACKET_TYPE_SOL_PAYLOAD_DATA_RQ)
-    str = "= SOL Remote Console to BMC                    =";
+    str_cmd = "SOL Remote Console to BMC";
   else if (p == IPMICONSOLE_PACKET_TYPE_SOL_PAYLOAD_DATA_RS)
-    str = "= SOL BMC to Remote Console                    =";
-  else if (p == IPMICONSOLE_PACKET_TYPE_GET_CHANNEL_PAYLOAD_VERSION_RQ)
-    str = "= Get Channel Payload Version Request          =";
-  else if (p == IPMICONSOLE_PACKET_TYPE_GET_CHANNEL_PAYLOAD_VERSION_RS)
-    str = "= Get Channel Payload Version Response         =";
-  else if (p == IPMICONSOLE_PACKET_TYPE_DEACTIVATE_PAYLOAD_RQ)
-    str = "= Deactivate Payload Request                   =";
-  else if (p == IPMICONSOLE_PACKET_TYPE_DEACTIVATE_PAYLOAD_RS)
-    str = "= Deactivate Payload Response                  =";
-  else if (p == IPMICONSOLE_PACKET_TYPE_CLOSE_SESSION_RQ)
-    str = "= Close Session Request                        =";
-  else if (p == IPMICONSOLE_PACKET_TYPE_CLOSE_SESSION_RS)
-    str = "= Close Session Response                       =";
+    str_cmd = "SOL BMC to Remote Console";
+  else if (p == IPMICONSOLE_PACKET_TYPE_GET_CHANNEL_PAYLOAD_VERSION_RQ
+           || p == IPMICONSOLE_PACKET_TYPE_GET_CHANNEL_PAYLOAD_VERSION_RS)
+    str_cmd = ipmi_cmd_str(IPMI_NET_FN_APP_RQ, IPMI_CMD_GET_CHANNEL_PAYLOAD_VERSION);
+  else if (p == IPMICONSOLE_PACKET_TYPE_DEACTIVATE_PAYLOAD_RQ
+           || p == IPMICONSOLE_PACKET_TYPE_DEACTIVATE_PAYLOAD_RS)
+    str_cmd = ipmi_cmd_str(IPMI_NET_FN_APP_RQ, IPMI_CMD_DEACTIVATE_PAYLOAD);
+  else if (p == IPMICONSOLE_PACKET_TYPE_CLOSE_SESSION_RQ
+           || p == IPMICONSOLE_PACKET_TYPE_CLOSE_SESSION_RS)
+    str_cmd = ipmi_cmd_str(IPMI_NET_FN_APP_RQ, IPMI_CMD_CLOSE_SESSION);
   else
     {
       IPMICONSOLE_CTX_DEBUG(c, ("invalid packet type: %d", p));
       ipmiconsole_ctx_set_errnum(c, IPMICONSOLE_ERR_INTERNAL_ERROR);
       return -1;
     }
+
+  /* special case: there is no "Request/Response" with SOL data */
+  if (p == IPMICONSOLE_PACKET_TYPE_SOL_PAYLOAD_DATA_RQ
+      || p == IPMICONSOLE_PACKET_TYPE_SOL_PAYLOAD_DATA_RS)
+    packet_direction = DEBUG_UTIL_DIRECTION_NONE;
+  else if (IPMICONSOLE_PACKET_TYPE_REQUEST(p))
+    packet_direction = DEBUG_UTIL_DIRECTION_REQUEST;
+  else 
+    packet_direction = DEBUG_UTIL_DIRECTION_RESPONSE;
   
-  if ((len = snprintf(hdr, hdrlen, fmt, str)) < 0)
+  if (debug_hdr_str(packet_type,
+                    packet_direction,
+                    str_cmd,
+                    hdr,
+                    hdrlen) < 0)
     {
-      IPMICONSOLE_CTX_DEBUG(c, ("snprintf: p = %d", p));
-      ipmiconsole_ctx_set_errnum(c, IPMICONSOLE_ERR_INTERNAL_ERROR);
-      return -1;
-    }
-  
-  if (len >= hdrlen)
-    {
-      IPMICONSOLE_CTX_DEBUG(c, ("snprintf truncation: p = %d; len = %d", p, len));
+      IPMICONSOLE_CTX_DEBUG(c, ("debug_hdr_str: p = %d", p));
       ipmiconsole_ctx_set_errnum(c, IPMICONSOLE_ERR_INTERNAL_ERROR);
       return -1;
     }
 
-  return (len);
+  return 0;
 }
 
 int
@@ -314,6 +317,7 @@ ipmiconsole_packet_dump(ipmiconsole_ctx_t c,
       if (ipmi_dump_lan_packet(fd,
 			       c->config.hostname,
 			       hdr,
+                               NULL,
 			       buf,
 			       buflen,
 			       tmpl_lan_msg_hdr,
@@ -339,6 +343,7 @@ ipmiconsole_packet_dump(ipmiconsole_ctx_t c,
 	  if (ipmi_dump_rmcpplus_packet(fd,
 					c->config.hostname,
 					hdr,
+                                        NULL,
 					IPMI_AUTHENTICATION_ALGORITHM_RAKP_NONE,
 					IPMI_INTEGRITY_ALGORITHM_NONE,
 					IPMI_CONFIDENTIALITY_ALGORITHM_NONE,
@@ -361,6 +366,7 @@ ipmiconsole_packet_dump(ipmiconsole_ctx_t c,
 	  if (ipmi_dump_rmcpplus_packet(fd,
 					c->config.hostname,
 					hdr,
+                                        NULL,
 					c->config.authentication_algorithm,
 					c->config.integrity_algorithm,
 					c->config.confidentiality_algorithm,
@@ -499,6 +505,7 @@ ipmiconsole_packet_dump_unknown(ipmiconsole_ctx_t c,
       if (ipmi_dump_lan_packet(fd,
 			       c->config.hostname,
 			       hdr,
+                               NULL,
 			       buf,
 			       buflen,
 			       tmpl_lan_msg_hdr_rs,
@@ -514,6 +521,7 @@ ipmiconsole_packet_dump_unknown(ipmiconsole_ctx_t c,
       if (ipmi_dump_rmcpplus_packet(fd,
 				    c->config.hostname,
 				    hdr,
+                                    NULL,
 				    c->config.authentication_algorithm,
 				    c->config.integrity_algorithm,
 				    c->config.confidentiality_algorithm,
@@ -581,7 +589,8 @@ _ipmi_1_5_packet_assemble(ipmiconsole_ctx_t c,
       return -1;
     }
 
-  if (fill_lan_msg_hdr(net_fn,
+  if (fill_lan_msg_hdr(IPMI_SLAVE_ADDRESS_BMC,
+		       net_fn,
 		       IPMI_BMC_IPMB_LUN_BMC,
 		       c->session.requester_sequence_number,
 		       c->connection.obj_lan_msg_hdr_rq) < 0)
@@ -668,7 +677,8 @@ _ipmi_2_0_packet_assemble(ipmiconsole_ctx_t c,
       return -1;
     }
       
-  if (fill_lan_msg_hdr(net_fn,
+  if (fill_lan_msg_hdr(IPMI_SLAVE_ADDRESS_BMC,
+		       net_fn,
 		       IPMI_BMC_IPMB_LUN_BMC,
 		       c->session.requester_sequence_number,
 		       c->connection.obj_lan_msg_hdr_rq) < 0)
@@ -950,24 +960,13 @@ ipmiconsole_ipmi_packet_assemble(ipmiconsole_ctx_t c,
       obj_cmd_rq = c->connection.obj_open_session_request;
     }
   else if (p == IPMICONSOLE_PACKET_TYPE_RAKP_MESSAGE_1)
-    {
-      uint8_t name_only_lookup;
-
-      /* IPMI Workaround
-       *
-       * Intel IPMI 2.0 implementations use this flag incorrectly.
-       */
-      if (c->config.workaround_flags & IPMICONSOLE_WORKAROUND_INTEL_2_0_SESSION)
-        name_only_lookup = IPMI_USER_NAME_PRIVILEGE_LOOKUP;
-      else
-        name_only_lookup = c->session.name_only_lookup;
-      
+    {     
       if (fill_rmcpplus_rakp_message_1 (c->session.message_tag,
 					managed_system_session_id,
 					c->session.remote_console_random_number,
 					IPMI_REMOTE_CONSOLE_RANDOM_NUMBER_LENGTH,
 					c->config.privilege_level,
-                                        name_only_lookup,
+                                        c->session.name_only_lookup,
 					username,
 					username_len,
 					c->connection.obj_rakp_message_1) < 0)
@@ -984,6 +983,16 @@ ipmiconsole_ipmi_packet_assemble(ipmiconsole_ctx_t c,
       int32_t managed_system_random_number_len;
       uint8_t key_exchange_authentication_code[IPMI_MAX_KEY_EXCHANGE_AUTHENTICATION_CODE_LENGTH];
       int32_t key_exchange_authentication_code_len;
+      uint8_t name_only_lookup;
+
+      /* IPMI Workaround
+       *
+       * Intel IPMI 2.0 implementations use this flag incorrectly.
+       */
+      if (c->config.workaround_flags & IPMICONSOLE_WORKAROUND_INTEL_2_0_SESSION)
+        name_only_lookup = IPMI_USER_NAME_PRIVILEGE_LOOKUP;
+      else
+        name_only_lookup = c->session.name_only_lookup;
       
       if ((managed_system_random_number_len = Fiid_obj_get_data(c,
 								c->connection.obj_rakp_message_2,
@@ -1004,7 +1013,7 @@ ipmiconsole_ipmi_packet_assemble(ipmiconsole_ctx_t c,
 													 managed_system_random_number,
 													 managed_system_random_number_len,
 													 c->session.remote_console_session_id,
-													 c->session.name_only_lookup,
+													 name_only_lookup,
 													 c->config.privilege_level,
 													 username,
                                                                                                          username_len,
@@ -1064,9 +1073,21 @@ ipmiconsole_ipmi_packet_assemble(ipmiconsole_ctx_t c,
     }
   else if (p == IPMICONSOLE_PACKET_TYPE_ACTIVATE_PAYLOAD_RQ)
     {
+      uint8_t sol_startup_handshake;
       uint8_t authentication_activation;
       uint8_t encryption_activation;
 
+      /* IPMI Workaround
+       *
+       * Intel IPMI 2.0 specific issue.  I have no idea why this is.
+       * I took it from ipmiutil.  Ipmiutil author Andrew Cress cannot
+       * remember why this is the case either.
+       */
+      if (c->config.workaround_flags & IPMICONSOLE_WORKAROUND_INTEL_2_0_SESSION)
+        sol_startup_handshake = IPMI_SOL_STARTUP_HANDSHAKE_CTS_AND_DCD_SDR_ASSERTED;
+      else
+        sol_startup_handshake = IPMI_SOL_STARTUP_HANDSHAKE_CTS_AND_DCD_SDR_DEASSERTED;
+
       if (c->config.integrity_algorithm == IPMI_INTEGRITY_ALGORITHM_NONE)
 	authentication_activation = IPMI_ACTIVATE_PAYLOAD_WITHOUT_AUTHENTICATION;
       else
@@ -1079,7 +1100,7 @@ ipmiconsole_ipmi_packet_assemble(ipmiconsole_ctx_t c,
 
       if (fill_cmd_activate_payload_sol(IPMI_PAYLOAD_TYPE_SOL,
 					c->session.sol_payload_instance,
-					IPMI_SOL_STARTUP_HANDSHAKE_CTS_AND_DCD_SDR_DEASSERTED, 
+					sol_startup_handshake, 
 					IPMI_SERIAL_MODEM_ALERTS_FAIL_WHILE_SOL_ACTIVE, 
 					IPMI_TEST_MODE_DEACTIVATED,
 					authentication_activation, 
@@ -1311,7 +1332,7 @@ ipmiconsole_packet_unassemble(ipmiconsole_ctx_t c,
 			      uint8_t *buf,
 			      uint32_t buflen)
 {
-  ipmiconsole_protocol_state_t pkt;
+  ipmiconsole_packet_type_t pkt;
   fiid_obj_t obj_cmd = NULL;
   int8_t rv;
 
