@@ -1,7 +1,7 @@
 /*****************************************************************************\
- *  $Id: ipmidetectd_loop.c,v 1.6 2007/10/18 16:18:48 chu11 Exp $
+ *  $Id: ipmidetectd_loop.c,v 1.11 2008/08/15 16:04:25 chu11 Exp $
  *****************************************************************************
- *  Copyright (C) 2007 Lawrence Livermore National Security, LLC.
+ *  Copyright (C) 2007-2008 Lawrence Livermore National Security, LLC.
  *  Copyright (C) 2007 The Regents of the University of California.
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
  *  Written by Albert Chu <chu11@llnl.gov>
@@ -69,6 +69,7 @@
 #include "debug.h"
 #include "error.h"
 #include "fd.h"
+#include "freeipmi-portability.h"
 #include "hash.h"
 #include "list.h"
 #include "timeval.h"
@@ -202,7 +203,13 @@ _nodes_setup(void)
       info->fd = fds[i/IPMIDETECTD_NODES_PER_SOCKET];
       
       if (!(h = gethostbyname(host)))
-        ERR_EXIT(("gethostbyname: %s", hstrerror(h_errno)));
+        {
+#if HAVE_HSTRERROR
+          ERR_EXIT(("gethostbyname: %s", hstrerror(h_errno)));
+#else /* !HAVE_HSTRERROR */
+          ERR_EXIT(("gethostbyname: h_errno = %d", h_errno));
+#endif /* !HAVE_HSTRERROR */
+        }
         
       info->destaddr.sin_family = AF_INET;
       info->destaddr.sin_addr = *((struct in_addr *)h->h_addr);
@@ -275,7 +282,8 @@ _ipmi_ping_build(struct ipmidetectd_info *info, char *buf, unsigned int buflen)
                            obj_lan_session_hdr) < 0)
     ERR_EXIT(("fill_lan_session_hdr: %s", strerror(errno)));
 
-  if (fill_lan_msg_hdr(IPMI_NET_FN_APP_RQ, 
+  if (fill_lan_msg_hdr(IPMI_SLAVE_ADDRESS_BMC,
+		       IPMI_NET_FN_APP_RQ, 
                        IPMI_BMC_IPMB_LUN_BMC,
                        info->sequence_number % (IPMI_RQ_SEQ_MAX+1), 
                        obj_lan_msg_hdr) < 0)
