@@ -1,7 +1,7 @@
 /*****************************************************************************\
- *  $Id: ipmi_monitoring.c,v 1.19 2007/10/18 16:18:49 chu11 Exp $
+ *  $Id: ipmi_monitoring.c,v 1.32.2.3 2008/12/09 18:42:14 chu11 Exp $
  *****************************************************************************
- *  Copyright (C) 2007 Lawrence Livermore National Security, LLC.
+ *  Copyright (C) 2007-2008 Lawrence Livermore National Security, LLC.
  *  Copyright (C) 2006-2007 The Regents of the University of California.
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
  *  Written by Albert Chu <chu11@llnl.gov>
@@ -37,7 +37,7 @@
 #include <sys/stat.h>
 #if HAVE_UNISTD_H
 #include <unistd.h>
-#endif /* !HAVE_UNISTD_H */
+#endif /* HAVE_UNISTD_H */
 #include <assert.h>
 #include <errno.h>
 
@@ -49,6 +49,7 @@
 #include "ipmi_monitoring_sensor_config.h"
 #include "ipmi_monitoring_sensor_reading.h"
 
+#include "freeipmi-portability.h"
 #include "secure.h"
 
 static char *ipmi_monitoring_errmsgs[] =
@@ -56,7 +57,7 @@ static char *ipmi_monitoring_errmsgs[] =
     "success",
     "ctx null",
     "ctx invalid",
-    "invalid parmaeters",
+    "invalid parameters",
     "permission denied",
     "library uninitialized",
     "config file parse error",
@@ -67,6 +68,7 @@ static char *ipmi_monitoring_errmsgs[] =
     "sensor not found",
     "no sensor readings available",
     "end of sensor readings list reached",
+    "connection timeout",
     "session timeout",
     "invalid username",
     "invalid password",
@@ -90,8 +92,14 @@ static int _ipmi_monitoring_initialized = 0;
 
 uint32_t _ipmi_monitoring_flags = 0;
 
-extern char sdr_cache_dir[MAXPATHLEN+1];
-extern int sdr_cache_dir_set;
+extern char sensor_config_file[MAXPATHLEN+1];
+extern int sensor_config_file_set;
+
+extern char sdr_cache_directory[MAXPATHLEN+1];
+extern int sdr_cache_directory_set;
+
+extern char sdr_cache_filename_format[MAXPATHLEN+1];
+extern int sdr_cache_filename_format_set;
 
 static void
 _init_ctx(ipmi_monitoring_ctx_t c)
@@ -210,16 +218,50 @@ ipmi_monitoring_init(unsigned int flags, int *errnum)
   if (_ipmi_monitoring_initialized)
     return 0;
 
+  /* before ipmi_monitoring_sensor_config() for debugging */
+  _ipmi_monitoring_flags = flags;
+
   if (ipmi_monitoring_sensor_config(errnum) < 0)
     return -1;
 
-  _ipmi_monitoring_flags = flags;
   _ipmi_monitoring_initialized++;
   if (errnum)
     *errnum = IPMI_MONITORING_ERR_SUCCESS;
   return 0;
 }
 
+int
+ipmi_monitoring_sensor_config_file(char *file, int *errnum)
+{
+  struct stat buf;
+
+  if (!file || (strlen(file) > MAXPATHLEN))
+    {
+      if (errnum)
+        *errnum = IPMI_MONITORING_ERR_PARAMETERS;
+      return -1;
+    }
+
+  if (stat(file, &buf) < 0)
+    {
+      if (errnum)
+        {
+          if (errno == EACCES || errno == EPERM)
+            *errnum = IPMI_MONITORING_ERR_PERMISSION;
+          else
+            *errnum = IPMI_MONITORING_ERR_PARAMETERS;
+        }
+      return -1;
+    }
+  
+  strncpy(sensor_config_file, file, MAXPATHLEN);
+  sensor_config_file_set = 1;
+
+  if (errnum)
+    *errnum = IPMI_MONITORING_ERR_SUCCESS;
+  return 0;
+}
+
 int 
 ipmi_monitoring_sdr_cache_directory(char *dir, int *errnum)
 {
@@ -244,8 +286,40 @@ ipmi_monitoring_sdr_cache_directory(char *dir, int *errnum)
       return -1;
     }
 
-  strncpy(sdr_cache_dir, dir, MAXPATHLEN);
-  sdr_cache_dir_set = 1;
+  strncpy(sdr_cache_directory, dir, MAXPATHLEN);
+  sdr_cache_directory_set = 1;
+
+  if (errnum)
+    *errnum = IPMI_MONITORING_ERR_SUCCESS;
+  return 0;
+}
+
+int 
+ipmi_monitoring_sdr_cache_filenames(char *format, int *errnum)
+{
+  if (!format)
+    {
+      if (errnum)
+        *errnum = IPMI_MONITORING_ERR_PARAMETERS;
+      return -1;
+    }
+
+  if (strchr(format, '/'))
+    {
+      if (errnum)
+        *errnum = IPMI_MONITORING_ERR_PARAMETERS;
+      return -1;
+    }
+
+  if (!strstr(format, "%H"))
+    {
+      if (errnum)
+        *errnum = IPMI_MONITORING_ERR_PARAMETERS;
+      return -1;
+    }
+
+  strncpy(sdr_cache_filename_format, format, MAXPATHLEN);
+  sdr_cache_filename_format_set = 1;
 
   if (errnum)
     *errnum = IPMI_MONITORING_ERR_SUCCESS;
@@ -334,7 +408,7 @@ ipmi_monitoring_sensor_readings_by_record_id(ipmi_monitoring_ctx_t c,
 
           if (ipmi_sdr_cache_search_record_id(c->sc, record_ids[i]) < 0)
             {
-              if (ipmi_sdr_cache_ctx_errnum(c->sc) == IPMI_SDR_CACHE_ERR_NOT_FOUND)
+              if (ipmi_sdr_cache_ctx_errnum(c->sc) == IPMI_SDR_CACHE_CTX_ERR_NOT_FOUND)
                 {
                   c->errnum = IPMI_MONITORING_ERR_SENSOR_NOT_FOUND;
                   goto cleanup;
@@ -723,7 +797,7 @@ ipmi_monitoring_bitmask_string(ipmi_monitoring_ctx_t c,
                                char *buffer,
                                unsigned int buflen)
 {
-  unsigned int offset;
+  unsigned int offset = 0;
   int i;
 
   if (!c || c->magic != IPMI_MONITORING_MAGIC)
@@ -748,6 +822,11 @@ ipmi_monitoring_bitmask_string(ipmi_monitoring_ctx_t c,
     {
       uint8_t event_reading_type_code;
 
+      /* achu: there are no "names" associated with
+       * event_reading_type_codes in the spec (table 42-2), so there are
+       * no macros.  We just gotta hard code numbers.
+       */
+
       switch (bitmask_type)
         {
         case IPMI_MONITORING_SENSOR_BITMASK_TYPE_TRANSITION:
@@ -802,10 +881,10 @@ ipmi_monitoring_bitmask_string(ipmi_monitoring_ctx_t c,
             }
         }
  
-      if (ipmi_get_generic_event_message (event_reading_type_code,
-                                          offset,
-                                          buffer,
-                                          buflen) < 0)
+      if (ipmi_get_generic_event_message_short (event_reading_type_code,
+                                                offset,
+                                                buffer,
+                                                buflen) < 0)
         {
           if (errno == EINVAL)
             c->errnum = IPMI_MONITORING_ERR_PARAMETERS;
@@ -821,46 +900,70 @@ ipmi_monitoring_bitmask_string(ipmi_monitoring_ctx_t c,
       switch (bitmask_type)
         {
         case IPMI_MONITORING_SENSOR_BITMASK_TYPE_PHYSICAL_SECURITY:
-          sensor_type_code = 0x05;
+          sensor_type_code = IPMI_SENSOR_TYPE_PHYSICAL_SECURITY;
           break;
         case IPMI_MONITORING_SENSOR_BITMASK_TYPE_PLATFORM_SECURITY_VIOLATION_ATTEMPT:
-          sensor_type_code = 0x06;
+          sensor_type_code = IPMI_SENSOR_TYPE_PLATFORM_SECURITY_VIOLATION_ATTEMPT;
           break;
         case IPMI_MONITORING_SENSOR_BITMASK_TYPE_PROCESSOR:
-          sensor_type_code = 0x07;
+          sensor_type_code = IPMI_SENSOR_TYPE_PROCESSOR;
           break;
         case IPMI_MONITORING_SENSOR_BITMASK_TYPE_POWER_SUPPLY:
-          sensor_type_code = 0x08;
+          sensor_type_code = IPMI_SENSOR_TYPE_POWER_SUPPLY;
           break;
         case IPMI_MONITORING_SENSOR_BITMASK_TYPE_POWER_UNIT:
-          sensor_type_code = 0x09;
+          sensor_type_code = IPMI_SENSOR_TYPE_POWER_UNIT;
           break;
         case IPMI_MONITORING_SENSOR_BITMASK_TYPE_MEMORY:
-          sensor_type_code = 0x0C;
+          sensor_type_code = IPMI_SENSOR_TYPE_MEMORY;
           break;
         case IPMI_MONITORING_SENSOR_BITMASK_TYPE_DRIVE_SLOT:
-          sensor_type_code = 0x0D;
+          sensor_type_code = IPMI_SENSOR_TYPE_DRIVE_SLOT;
           break;
         case IPMI_MONITORING_SENSOR_BITMASK_TYPE_SYSTEM_FIRMWARE_PROGRESS:
-          sensor_type_code = 0x0F;
+          sensor_type_code = IPMI_SENSOR_TYPE_SYSTEM_FIRMWARE_PROGRESS;
           break;
         case IPMI_MONITORING_SENSOR_BITMASK_TYPE_EVENT_LOGGING_DISABLED:
-          sensor_type_code = 0x10;
+          sensor_type_code = IPMI_SENSOR_TYPE_EVENT_LOGGING_DISABLED;
           break;
         case IPMI_MONITORING_SENSOR_BITMASK_TYPE_SYSTEM_EVENT:
-          sensor_type_code = 0x12;
+          sensor_type_code = IPMI_SENSOR_TYPE_SYSTEM_EVENT;
           break;
         case IPMI_MONITORING_SENSOR_BITMASK_TYPE_CRITICAL_INTERRUPT:
-          sensor_type_code = 0x13;
+          sensor_type_code = IPMI_SENSOR_TYPE_CRITICAL_INTERRUPT;
           break;
         case IPMI_MONITORING_SENSOR_BITMASK_TYPE_MODULE_BOARD:
-          sensor_type_code = 0x15;
+          sensor_type_code = IPMI_SENSOR_TYPE_MODULE_BOARD;
           break;
         case IPMI_MONITORING_SENSOR_BITMASK_TYPE_SLOT_CONNECTOR:
-          sensor_type_code = 0x21;
+          sensor_type_code = IPMI_SENSOR_TYPE_SLOT_CONNECTOR;
           break;
         case IPMI_MONITORING_SENSOR_BITMASK_TYPE_WATCHDOG2:
-          sensor_type_code = 0x23;
+          sensor_type_code = IPMI_SENSOR_TYPE_WATCHDOG2;
+          break;
+        case IPMI_MONITORING_SENSOR_BITMASK_TYPE_ENTITY_PRESENCE:
+          sensor_type_code = IPMI_SENSOR_TYPE_ENTITY_PRESENCE;
+          break;
+        case IPMI_MONITORING_SENSOR_BITMASK_TYPE_MANAGEMENT_SUBSYSTEM_HEALTH:
+          sensor_type_code = IPMI_SENSOR_TYPE_MANAGEMENT_SUBSYSTEM_HEALTH;
+          break;
+        case IPMI_MONITORING_SENSOR_BITMASK_TYPE_BATTERY:
+          sensor_type_code = IPMI_SENSOR_TYPE_BATTERY;
+          break;
+        case IPMI_MONITORING_SENSOR_BITMASK_TYPE_FRU_STATE:
+          sensor_type_code = IPMI_SENSOR_TYPE_FRU_STATE;
+          break;
+        case IPMI_MONITORING_SENSOR_BITMASK_TYPE_CABLE_INTERCONNECT:
+          sensor_type_code = IPMI_SENSOR_TYPE_CABLE_INTERCONNECT;
+          break;
+        case IPMI_MONITORING_SENSOR_BITMASK_TYPE_BOOT_ERROR:
+          sensor_type_code = IPMI_SENSOR_TYPE_BOOT_ERROR;
+          break;
+        case IPMI_MONITORING_SENSOR_BITMASK_TYPE_BUTTON_SWITCH:
+          sensor_type_code = IPMI_SENSOR_TYPE_BUTTON_SWITCH;
+          break;
+        case IPMI_MONITORING_SENSOR_BITMASK_TYPE_SYSTEM_ACPI_POWER_STATE:
+          sensor_type_code = IPMI_SENSOR_TYPE_SYSTEM_ACPI_POWER_STATE;
           break;
         default:
           c->errnum = IPMI_MONITORING_ERR_INTERNAL_ERROR;
@@ -881,10 +984,10 @@ ipmi_monitoring_bitmask_string(ipmi_monitoring_ctx_t c,
             }
         }
 
-      if (ipmi_get_sensor_type_code_message (sensor_type_code,
-                                             offset,
-                                             buffer,
-                                             buflen) < 0)
+      if (ipmi_get_sensor_type_code_message_short (sensor_type_code,
+                                                   offset,
+                                                   buffer,
+                                                   buflen) < 0)
         {
           if (errno == EINVAL)
             c->errnum = IPMI_MONITORING_ERR_PARAMETERS;
