@@ -1,7 +1,7 @@
 /*****************************************************************************\
- *  $Id: ipmipower.c,v 1.41 2007/11/20 17:51:35 chu11 Exp $
+ *  $Id: ipmipower.c,v 1.72 2008/08/12 18:14:40 chu11 Exp $
  *****************************************************************************
- *  Copyright (C) 2007 Lawrence Livermore National Security, LLC.
+ *  Copyright (C) 2007-2008 Lawrence Livermore National Security, LLC.
  *  Copyright (C) 2003-2007 The Regents of the University of California.
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
  *  Written by Albert Chu <chu11@llnl.gov>
@@ -36,7 +36,7 @@
 #endif /* HAVE_UNISTD_H */
 #if STDC_HEADERS
 #include <string.h>
-#endif /* STDC_HEADERS_H */
+#endif /* STDC_HEADERS */
 #if TIME_WITH_SYS_TIME
 #include <sys/time.h>
 #include <time.h>
@@ -56,7 +56,7 @@
 #include <errno.h>
 
 #include "ipmipower.h"
-#include "ipmipower_config.h"
+#include "ipmipower_argp.h"
 #include "ipmipower_connection.h"
 #include "ipmipower_powercmd.h"
 #include "ipmipower_prompt.h"
@@ -64,43 +64,24 @@
 #include "ipmipower_util.h"
 #include "ipmipower_wrappers.h"
 
+#include "freeipmi-portability.h"
+#include "tool-common.h"
+
 cbuf_t ttyin;
 cbuf_t ttyout;
 cbuf_t ttyerr;
 
 /* configuration for ipmipower */
-struct ipmipower_config *conf = NULL;
+struct ipmipower_arguments cmd_args;
 
 /* Array of all ipmi "connections" */
 struct ipmipower_connection *ics = NULL;
+unsigned int ics_len = 0;
 
 /* Array of hostlists for short output */
 int output_hostrange_flag = 0;
 hostlist_t output_hostrange[MSG_TYPE_NUM_ENTRIES];
 
-/* 
- * _security_initialization
- */
-static void
-_security_initialization(void)
-{
-  /* Disable core dumping when not-debugging.  Do not want username,
-   * password or other important stuff to core dump.
-   */
-#ifdef NDEBUG
-  struct rlimit rlim;
-
-  if (getrlimit(RLIMIT_CORE, &rlim) < 0)
-    err_exit("getrlimit: %s", strerror(errno));
-
-  rlim.rlim_cur = 0;
-  if (setrlimit(RLIMIT_CORE,&rlim) < 0)
-    err_exit("setrlimit: %s", strerror(errno));
-#endif /* NDEBUG */
-  if (ipmi_rmcpplus_init() < 0)
-    err_exit("ipmi_rmcpplus_init");
-}
-
 /* _setup
  * - Setup structures and values for the program
  */
@@ -127,40 +108,23 @@ _setup(void)
   ttyout = Cbuf_create(IPMIPOWER_MIN_TTY_BUF, IPMIPOWER_MAX_TTY_BUF);
   ttyerr = Cbuf_create(IPMIPOWER_MIN_TTY_BUF, IPMIPOWER_MAX_TTY_BUF);
 
-  if (conf->hosts != NULL) 
-    {
-      ics = ipmipower_connection_array_create(conf->hosts, conf->hosts_count);
-      if (ics == NULL)
-        exit(1);		/* error message output in the above call */
-    }
-
   for (i = 0; i < MSG_TYPE_NUM_ENTRIES; i++) 
     {
-      if ((output_hostrange[i] = hostlist_create(NULL)) == NULL)
-        err_exit("hostlist_create() error");
+      if (!(output_hostrange[i] = hostlist_create(NULL)))
+        ierr_exit("hostlist_create() error");
     }
   
-  /* errors should always go to atleast the syslog */
-  err_syslog(1);
+  /* in interactive mode errors should always go to atleast the syslog */
+  ierr_syslog(1);
 
 #ifndef NDEBUG
-  if (strlen(conf->logfile)) 
-    {
-      conf->logfile_fd = open(conf->logfile, O_WRONLY | O_CREAT | O_APPEND,
-                              S_IRUSR | S_IWUSR);
-      if (conf->logfile_fd < 0)
-        err_exit("error opening log file %s: %s", conf->logfile, strerror(errno));
-    }
-
   /* if debug set, send debug info to stderr too */
-  err_cbuf(conf->debug, ttyerr);
-
-  /* on err_exit() dump cbuf data to appropriate places too */
-  err_cbuf_dump_file_stream(conf->debug, stderr);
-  err_cbuf_dump_file_descriptor(conf->log, conf->logfile_fd);
+  ierr_cbuf(cmd_args.common.debug, ttyerr);
 
+  /* on ierr_exit() dump cbuf data to appropriate places too */
+  ierr_cbuf_dump_file_stream(cmd_args.common.debug, stderr);
 #else  /* !NDEBUG */
-  err_cbuf(0, 0);
+  ierr_cbuf(0, 0);
 #endif /* !NDEBUG */
 }
 
@@ -175,74 +139,54 @@ _cleanup(void)
   cbuf_destroy(ttyin);
   
   /* Flush before destroying. */
-#ifndef NDEBUG
-  if (conf->log)
-    cbuf_peek_to_fd(ttyout, conf->logfile_fd, -1);
-#endif /* NDEBUG */
   cbuf_read_to_fd(ttyout, STDOUT_FILENO, -1);
   cbuf_destroy(ttyout);
-#ifndef NDEBUG
-  if (conf->log)
-    cbuf_peek_to_fd(ttyerr, conf->logfile_fd, -1);
-#endif /* NDEBUG */
   cbuf_read_to_fd(ttyerr, STDERR_FILENO, -1);
   cbuf_destroy(ttyerr);
 
-  ipmipower_connection_array_destroy(ics, conf->hosts_count);
+  ipmipower_connection_array_destroy(ics, ics_len);
 
   for (i = 0; i < MSG_TYPE_NUM_ENTRIES; i++)
     hostlist_destroy(output_hostrange[i]);
-
-  hostlist_destroy(conf->hosts);
-
-#ifndef NDEBUG
-  close(conf->logfile_fd);
-#endif /* NDEBUG */
-
-#ifdef NDEBUG
-  secure_free(conf, sizeof(struct ipmipower_config));
-#else  /* !NDEBUG */
-  free(conf);
-#endif /* !NDEBUG */
 }
 
 static void 
 _sendto(cbuf_t buf, int fd, struct sockaddr_in *destaddr) 
 {
   int n, rv;
-  char buffer[IPMI_PACKET_BUFLEN];
+  char buffer[IPMIPOWER_PACKET_BUFLEN];
 
-  if ((n = cbuf_read(buf, buffer, IPMI_PACKET_BUFLEN)) < 0)
-    err_exit("_sendto(%d): cbuf_read: %s", fd, strerror(errno));
-  if (n == IPMI_PACKET_BUFLEN)
-    err_exit("_sendto: Buffer full");
+  if ((n = cbuf_read(buf, buffer, IPMIPOWER_PACKET_BUFLEN)) < 0)
+    ierr_exit("_sendto(%d): cbuf_read: %s", fd, strerror(errno));
+  if (n == IPMIPOWER_PACKET_BUFLEN)
+    ierr_exit("_sendto: Buffer full");
     
   rv = ipmi_lan_sendto(fd, buffer, n, 0, (struct sockaddr *)destaddr,
                        sizeof(struct sockaddr_in));
   if (rv < 0)
-    err_exit("_sendto: ipmi_lan_sendto %s", strerror(errno));
+    ierr_exit("_sendto: ipmi_lan_sendto %s", strerror(errno));
   if (rv != n)
-    err_exit("_sendto: ipmi_lan_sendto rv=%d n=%d", rv, n);
+    ierr_exit("_sendto: ipmi_lan_sendto rv=%d n=%d", rv, n);
   
   /* cbuf should be empty now */
   if (!cbuf_is_empty(buf))
-    err_exit("_sendto: cbuf not empty");
+    ierr_exit("_sendto: cbuf not empty");
 }
 
 static void 
 _recvfrom(cbuf_t buf, int fd, struct sockaddr_in *srcaddr) 
 {
   int n, rv, dropped = 0;
-  char buffer[IPMI_PACKET_BUFLEN];
+  char buffer[IPMIPOWER_PACKET_BUFLEN];
   struct sockaddr_in from;
   unsigned int fromlen = sizeof(struct sockaddr_in);
   
-  rv = ipmi_lan_recvfrom(fd, buffer, IPMI_PACKET_BUFLEN, 0, 
+  rv = ipmi_lan_recvfrom(fd, buffer, IPMIPOWER_PACKET_BUFLEN, 0, 
 			 (struct sockaddr *)&from, &fromlen);
   if (rv < 0)
-    err_exit("_recvfrom: ipmi_lan_recvfrom: %s", strerror(errno));
-  if (rv == 0)
-    err_exit("_recvfrom: ipmi_lan_recvfrom: EOF");
+    ierr_exit("_recvfrom: ipmi_lan_recvfrom: %s", strerror(errno));
+  if (!rv)
+    ierr_exit("_recvfrom: ipmi_lan_recvfrom: EOF");
   
   /* Don't store if this is packet is strange for some reason */
   if (from.sin_family != AF_INET 
@@ -252,20 +196,21 @@ _recvfrom(cbuf_t buf, int fd, struct sockaddr_in *srcaddr)
   /* cbuf should be empty, but if it isn't, empty it */
   if (!cbuf_is_empty(buf)) 
     {
-      err_output("_recvfrom: cbuf not empty, draining");
-      do {
-        char tempbuf[IPMI_PACKET_BUFLEN];
-        if (cbuf_read(buf, tempbuf, IPMI_PACKET_BUFLEN) < 0)
-            err_exit("_recvfrom: cbuf_read: %s", strerror(errno));
-      } while(!cbuf_is_empty(buf));
+      ierr_dbg("_recvfrom: cbuf not empty, draining");
+      do 
+        {
+          char tempbuf[IPMIPOWER_PACKET_BUFLEN];
+          if (cbuf_read(buf, tempbuf, IPMIPOWER_PACKET_BUFLEN) < 0)
+            ierr_exit("_recvfrom: cbuf_read: %s", strerror(errno));
+        } while(!cbuf_is_empty(buf));
     }
 
   if ((n = cbuf_write(buf, buffer, rv, &dropped)) < 0)
-    err_exit("_recvfrom(%d): cbuf_write: %s", fd, strerror(errno));
+    ierr_exit("_recvfrom(%d): cbuf_write: %s", fd, strerror(errno));
   if (n != rv)
-    err_exit("_recvfrom: rv=%d n=%d", rv, n);
-  if (dropped != 0)
-    err_output("_recvfrom: read dropped %d bytes", dropped);
+    ierr_exit("_recvfrom: rv=%d n=%d", rv, n);
+  if (dropped)
+    ierr_dbg("_recvfrom: read dropped %d bytes", dropped);
 }
 
 /* _poll_loop
@@ -287,13 +232,13 @@ _poll_loop(int non_interactive)
        * powercmd_timeout will not be set, leaving it at -1
        */
       num = ipmipower_powercmd_process_pending(&powercmd_timeout);
-      if (non_interactive && num == 0)
+      if (non_interactive && !num)
         break;
 
-      /* ping timeout is always set if conf->ping_interval_len > 0 */
+      /* ping timeout is always set if cmd_args.ping_interval > 0 */
       ipmipower_ping_process_pings(&ping_timeout);
 
-      if (conf->ping_interval_len) 
+      if (cmd_args.ping_interval) 
         {
           if (powercmd_timeout == -1)
             timeout = ping_timeout;
@@ -318,18 +263,18 @@ _poll_loop(int non_interactive)
        */
 
       /* Has the number of hosts changed? */
-      if (nfds != (conf->hosts_count*2) + 3)
+      if (nfds != (ics_len*2) + 3)
 	{
 	  /* The "*2" is for each host's two fds, one for ipmi
 	   * (ipmi_fd) and one for rmcp (ping_fd).  The "+3" is for
 	   * stdin, stdout, stderr.
 	   */
-	  nfds = (conf->hosts_count*2) + 3;   
+	  nfds = (ics_len*2) + 3;   
 	  Free(pfds);
 	  pfds = (struct pollfd *)Malloc(nfds * sizeof(struct pollfd));
 	}
       
-      for (i = 0; i < conf->hosts_count; i++) 
+      for (i = 0; i < ics_len; i++) 
         {
           pfds[i*2].fd = ics[i].ipmi_fd;
           pfds[i*2+1].fd = ics[i].ping_fd;
@@ -340,7 +285,7 @@ _poll_loop(int non_interactive)
           if (!cbuf_is_empty(ics[i].ipmi_out))
             pfds[i*2].events |= POLLOUT;
           
-          if (!conf->ping_interval_len)
+          if (!cmd_args.ping_interval)
             continue;
           
           pfds[i*2+1].events |= POLLIN;
@@ -366,12 +311,12 @@ _poll_loop(int non_interactive)
 
       Poll(pfds, nfds, timeout);
       
-      for (i = 0; i < conf->hosts_count; i++) 
+      for (i = 0; i < ics_len; i++) 
         {
           if (pfds[i*2].revents & POLLERR) 
             {
-              err_output("_poll_loop: IPMI POLLERR, %s, %d", ics[i].hostname, 
-                         ics[i].ipmi_fd);
+              ierr_dbg("_poll_loop: IPMI POLLERR, %s, %d", ics[i].hostname, 
+                       ics[i].ipmi_fd);
               continue;
             }
           if (pfds[i*2].revents & POLLIN) 
@@ -379,13 +324,13 @@ _poll_loop(int non_interactive)
           if (pfds[i*2].revents & POLLOUT)
             _sendto(ics[i].ipmi_out, ics[i].ipmi_fd, &(ics[i].destaddr));
           
-          if (!conf->ping_interval_len)
+          if (!cmd_args.ping_interval)
             continue;
           
           if (pfds[i*2+1].revents & POLLERR) 
             {
-              err_output("_poll_loop: PING POLLERR, %s, %d", ics[i].hostname, 
-                         ics[i].ipmi_fd);
+              ierr_dbg("_poll_loop: PING POLLERR, %s, %d", ics[i].hostname, 
+                       ics[i].ipmi_fd);
               continue;
             }
           if (pfds[i*2+1].revents & POLLIN) 
@@ -395,29 +340,11 @@ _poll_loop(int non_interactive)
         }
       
       if (pfds[nfds-3].revents & POLLIN)
-        {
-          Cbuf_write_from_fd(ttyin, STDIN_FILENO);
-#ifndef NDEBUG
-          if (conf->log)
-            Cbuf_peek_to_fd(ttyin, conf->logfile_fd, -1);
-#endif /* NDEBUG */
-        }
+        Cbuf_write_from_fd(ttyin, STDIN_FILENO);
       if (!cbuf_is_empty(ttyout) && (pfds[nfds-2].revents & POLLOUT))
-        {
-#ifndef NDEBUG
-          if (conf->log)
-            Cbuf_peek_to_fd(ttyout, conf->logfile_fd, -1);
-#endif /* NDEBUG */
-          Cbuf_read_to_fd(ttyout, STDOUT_FILENO);
-        }
+        Cbuf_read_to_fd(ttyout, STDOUT_FILENO);
       if (!cbuf_is_empty(ttyerr) && (pfds[nfds-1].revents & POLLOUT))
-        {
-#ifndef NDEBUG
-          if (conf->log)
-            Cbuf_peek_to_fd(ttyerr, conf->logfile_fd, -1);
-#endif /* NDEBUG */
-          Cbuf_read_to_fd(ttyerr, STDERR_FILENO);
-        }
+        Cbuf_read_to_fd(ttyerr, STDERR_FILENO);
     }
 
   Free(pfds);
@@ -426,14 +353,13 @@ _poll_loop(int non_interactive)
 static void
 _eliminate_nodes(void)
 {
-  if (conf->eliminate == IPMIPOWER_TRUE)
+  if (cmd_args.hostrange.eliminate)
     {
       ipmidetect_t id = NULL;
-      hostlist_iterator_t itr = NULL;
-      char *host = NULL;
+      int i;
 
       if (!(id = ipmidetect_handle_create()))
-        err_exit("ipmidetect_handle_create");
+        ierr_exit("ipmidetect_handle_create");
       
       if (ipmidetect_load_data(id,
                                NULL,
@@ -442,32 +368,26 @@ _eliminate_nodes(void)
         {
           if (ipmidetect_errnum(id) == IPMIDETECT_ERR_CONNECT
               || ipmidetect_errnum(id) == IPMIDETECT_ERR_CONNECT_TIMEOUT)
-            err_exit("Error connecting to ipmidetect daemon");
-          err_exit("ipmidetect_load_data: %s\n", ipmidetect_errormsg(id));
+            ierr_exit("Error connecting to ipmidetect daemon");
+          ierr_exit("ipmidetect_load_data: %s", ipmidetect_errormsg(id));
         }
       
-      if (!(itr = hostlist_iterator_create(conf->hosts)))
-        err_exit("hostlist_iterator_create: %s", strerror(errno));
-      
-      while ((host = hostlist_next(itr)))
+      for (i = 0; i < ics_len; i++)
         {
           int ret;
           
-          if ((ret = ipmidetect_is_node_detected(id, host)) < 0)
+          if ((ret = ipmidetect_is_node_detected(id, ics[i].hostname)) < 0)
             {
               if (ipmidetect_errnum(id) == IPMIDETECT_ERR_NOTFOUND)
-                err_exit("Node '%s' unrecognized by ipmidetect\n", host);
-              err_exit("ipmidetect_is_node_detected: %s\n", ipmidetect_errormsg(id));
+                ierr_exit("Node '%s' unrecognized by ipmidetect", ics[i].hostname);
+              ierr_exit("ipmidetect_is_node_detected: %s", ipmidetect_errormsg(id));
             }
-          
+
           if (!ret)
-            {
-              hostlist_delete(conf->hosts, host);
-              conf->hosts_count--;
-            }
-          
-          free(host);
+            ics[i].skip++;
         }
+      
+      ipmidetect_handle_destroy(id);
     }
 }
 
@@ -475,53 +395,64 @@ int
 main(int argc, char *argv[]) 
 {
   /* Call before anything else */
-  err_init(argv[0]);
-  err_file_descriptor(1, STDERR_FILENO); /* initially errors goto stderr */
+  ierr_init(argv[0]);
+  ierr_file_descriptor(1, STDERR_FILENO); /* initially errors goto stderr */
   
-  _security_initialization();
+  ipmi_disable_coredump();
 
-  ipmipower_config_setup();
-  assert(conf != NULL);
+  if (ipmi_rmcpplus_init() < 0)
+    ierr_exit("ipmi_rmcpplus_init");
 
-  /* Must be called before conffile_parse b/c --config option on command line*/
-  ipmipower_config_cmdline_parse(argc, argv);
-    
-  ipmipower_config_conffile_parse(conf->configfile);
-  ipmipower_config_check_values();
+  ipmipower_argp_parse (argc, argv, &cmd_args);
 
   _setup();
 
   ipmipower_powercmd_setup();
   
+  if (cmd_args.common.hostname) 
+    {
+      unsigned int len = 0;
+      if (!(ics = ipmipower_connection_array_create(cmd_args.common.hostname, &len)))
+        {
+          /* dump error outputs here, most notably invalid hostname output */
+          if (cbuf_read_to_fd(ttyout, STDOUT_FILENO, -1) > 0)
+            exit(1);
+          ierr_exit("ipmipower_connection_array_create: %s", strerror(errno));
+        }
+      ics_len = len;
+    }
+  
   /* If power command (i.e. --reset, --stat, etc.) is passed at
    * command line, put the power control commands in the pending
    * queue.
    */
-  if (conf->powercmd != POWER_CMD_NONE) 
+  if (cmd_args.powercmd != POWER_CMD_NONE) 
     {
       int i;
 
+      cmd_args.ping_interval = 0;
+
       /* Check for appropriate privilege first */
-      if (conf->privilege_level_set_on_cmdline
-          && conf->privilege_level == PRIVILEGE_LEVEL_USER 
-          && POWER_CMD_REQUIRES_OPERATOR_PRIVILEGE_LEVEL(conf->powercmd))
-        err_exit("power operation requires atleast operator privilege");
+      if (cmd_args.common.privilege_level == IPMI_PRIVILEGE_LEVEL_USER 
+          && POWER_CMD_REQUIRES_OPERATOR_PRIVILEGE_LEVEL(cmd_args.powercmd))
+        ierr_exit("power operation requires atleast operator privilege");
 
       _eliminate_nodes();
 
-      for (i = 0; i < conf->hosts_count; i++) 
+      for (i = 0; i < ics_len; i++) 
         {
-          ipmipower_connection_clear(&ics[i]);
-          ipmipower_powercmd_queue(conf->powercmd, &ics[i]);
+          if (ics[i].skip)
+            continue;
+          ipmipower_powercmd_queue(cmd_args.powercmd, &ics[i]);
         }
     }
   
-  err_file_descriptor(0, 0); /* now errors are done through the ttyerr */
+  ierr_file_descriptor(0, 0); /* now errors are done through the ttyerr */
 
   /* immediately send out discovery messages upon startup */
   ipmipower_ping_force_discovery_sweep();
 
-  _poll_loop((conf->powercmd != POWER_CMD_NONE) ? 1 : 0);
+  _poll_loop((cmd_args.powercmd != POWER_CMD_NONE) ? 1 : 0);
   
   ipmipower_powercmd_cleanup();
   _cleanup();
