@@ -1,7 +1,7 @@
 /*****************************************************************************\
- *  $Id: ipmipower_check.c,v 1.68 2007/10/18 16:18:51 chu11 Exp $
+ *  $Id: ipmipower_check.c,v 1.89.4.4 2008/12/12 18:55:51 chu11 Exp $
  *****************************************************************************
- *  Copyright (C) 2007 Lawrence Livermore National Security, LLC.
+ *  Copyright (C) 2007-2008 Lawrence Livermore National Security, LLC.
  *  Copyright (C) 2003-2007 The Regents of the University of California.
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
  *  Written by Albert Chu <chu11@llnl.gov>
@@ -36,11 +36,16 @@
 #include <errno.h>
 #include <assert.h>
 
+#include "freeipmi-portability.h"
+
 #include "ipmipower_check.h"
 #include "ipmipower_packet.h"
 #include "ipmipower_wrappers.h"      
 
-extern struct ipmipower_config *conf;
+extern struct ipmipower_arguments cmd_args;
+
+#define IPMIPOWER_SEQUENCE_NUMBER_WINDOW 8
+#define IPMIPOWER_MAX_SEQUENCE_NUMBER    0xFFFFFFFF
 
 int
 ipmipower_check_checksum(ipmipower_powercmd_t ip, packet_type_t pkt)
@@ -48,13 +53,12 @@ ipmipower_check_checksum(ipmipower_powercmd_t ip, packet_type_t pkt)
   fiid_obj_t obj_cmd;
   int8_t rv;
 
-  assert(ip != NULL);
+  assert(ip);
   assert(PACKET_TYPE_VALID_RES(pkt));
   assert (pkt == AUTHENTICATION_CAPABILITIES_V20_RES
 	  || pkt == AUTHENTICATION_CAPABILITIES_RES
 	  || pkt == GET_SESSION_CHALLENGE_RES
 	  || pkt == ACTIVATE_SESSION_RES
-	  || pkt == GET_CHANNEL_CIPHER_SUITES_RES
 	  || pkt == SET_SESSION_PRIVILEGE_LEVEL_RES /* IPMI 1.5 or 2.0 */
 	  || pkt == GET_CHASSIS_STATUS_RES /* IPMI 1.5 or 2.0 */
 	  || pkt == CHASSIS_CONTROL_RES /* IPMI 1.5 or 2.0 */
@@ -64,13 +68,13 @@ ipmipower_check_checksum(ipmipower_powercmd_t ip, packet_type_t pkt)
   if ((rv = ipmi_lan_check_checksum(ip->obj_lan_msg_hdr_res,
 				    obj_cmd,
 				    ip->obj_lan_msg_trlr_res)) < 0)
-    err_exit("ipmipower_check_checksum(%s:%d): "
-	     "ipmi_lan_check_checksum: %s",
-	     ip->ic->hostname, ip->protocol_state, strerror(errno));
+    ierr_exit("ipmipower_check_checksum(%s:%d): "
+              "ipmi_lan_check_checksum: %s",
+              ip->ic->hostname, ip->protocol_state, strerror(errno));
 
   if (!rv)
-    dbg("ipmipower_check_checksum(%s:%d): checksum check failed",
-        ip->ic->hostname, ip->protocol_state);
+    ierr_dbg("ipmipower_check_checksum(%s:%d): checksum check failed",
+             ip->ic->hostname, ip->protocol_state);
 
   return (int)rv;
 }
@@ -84,13 +88,12 @@ ipmipower_check_authentication_code(ipmipower_powercmd_t ip,
   char *password;
   int8_t rv = -1;
 
-  assert(ip != NULL);
+  assert(ip);
   assert(PACKET_TYPE_VALID_RES(pkt));
   assert(pkt == AUTHENTICATION_CAPABILITIES_V20_RES
 	 || pkt == AUTHENTICATION_CAPABILITIES_RES
 	 || pkt == GET_SESSION_CHALLENGE_RES
 	 || pkt == ACTIVATE_SESSION_RES
-	 || pkt == GET_CHANNEL_CIPHER_SUITES_RES
 	 || pkt == SET_SESSION_PRIVILEGE_LEVEL_RES /* IPMI 1.5 or 2.0 */
 	 || pkt == GET_CHASSIS_STATUS_RES /* IPMI 1.5 or 2.0 */
 	 || pkt == CHASSIS_CONTROL_RES /* IPMI 1.5 or 2.0 */
@@ -101,7 +104,7 @@ ipmipower_check_authentication_code(ipmipower_powercmd_t ip,
       || pkt == AUTHENTICATION_CAPABILITIES_RES
       || pkt == GET_SESSION_CHALLENGE_RES
       || pkt == ACTIVATE_SESSION_RES
-      || (ip->ipmi_version == IPMI_VERSION_1_5
+      || (cmd_args.common.driver_type == IPMI_DEVICE_LAN
 	  && (pkt == SET_SESSION_PRIVILEGE_LEVEL_RES
 	      || pkt == GET_CHASSIS_STATUS_RES
 	      || pkt == CHASSIS_CONTROL_RES
@@ -113,33 +116,27 @@ ipmipower_check_authentication_code(ipmipower_powercmd_t ip,
       /* IPMI 1.5 Checks */
       if (pkt == AUTHENTICATION_CAPABILITIES_V20_RES
           || pkt == AUTHENTICATION_CAPABILITIES_RES
-          || pkt == GET_SESSION_CHALLENGE_RES
-	  || pkt == GET_CHANNEL_CIPHER_SUITES_RES)
+          || pkt == GET_SESSION_CHALLENGE_RES)
 	authentication_type = IPMI_AUTHENTICATION_TYPE_NONE;
       else if (pkt == ACTIVATE_SESSION_RES)
-	authentication_type = ip->authentication_type;
+	authentication_type = cmd_args.common.authentication_type;
       else /* pkt == SET_SESSION_PRIVILEGE_LEVEL_RES
               || pkt == GET_CHASSIS_STATUS_RES
               || pkt == CHASSIS_CONTROL_RES
               || pkt == CLOSE_SESSION_RES
 	   */
 	{
-	  if (ip->permsgauth_enabled == IPMIPOWER_FALSE)
+	  if (!ip->permsgauth_enabled)
 	    {
 	      authentication_type = IPMI_AUTHENTICATION_TYPE_NONE;
 	      check_authcode_retry_flag++;
 	    }
 	  else
-	    authentication_type = ip->authentication_type;
+	    authentication_type = cmd_args.common.authentication_type;
 	}
       
       if (authentication_type != IPMI_AUTHENTICATION_TYPE_NONE)
-	{
-	  if (strlen(conf->password))
-	    password = conf->password;
-	  else
-	    password = NULL;
-	}
+        password = cmd_args.common.password;
       else
 	password = NULL;
       
@@ -147,10 +144,10 @@ ipmipower_check_authentication_code(ipmipower_powercmd_t ip,
 								  buffer_len,
 								  authentication_type,
 								  (uint8_t *)password,
-								  strlen(conf->password))) < 0)
-	err_exit("ipmipower_check_authentication_code(%s:%d): "
-		 "ipmi_lan_check_packet_session_authentication_code: %s",
-		 ip->ic->hostname, ip->protocol_state, strerror(errno));
+								  password ? strlen(password) : 0)) < 0)
+	ierr_exit("ipmipower_check_authentication_code(%s:%d): "
+                  "ipmi_lan_check_packet_session_authentication_code: %s",
+                  ip->ic->hostname, ip->protocol_state, strerror(errno));
       
       /* IPMI Workaround (achu)
        *
@@ -163,21 +160,16 @@ ipmipower_check_authentication_code(ipmipower_powercmd_t ip,
        * here is our second session-authcode check attempt under these
        * circumstances.
        */
-      if ((conf->workaround_flags & WORKAROUND_FLAG_CHECK_UNEXPECTED_AUTHCODE)
+      if ((cmd_args.common.workaround_flags & IPMI_TOOL_WORKAROUND_FLAGS_CHECK_UNEXPECTED_AUTHCODE)
 	  && !rv
 	  && check_authcode_retry_flag)
 	{
-	  dbg("ipmipower_check_authentication_code(%s:%d): retry authcode check",
-	      ip->ic->hostname, ip->protocol_state, strerror(errno));
+	  ierr_dbg("ipmipower_check_authentication_code(%s:%d): retry authcode check",
+                   ip->ic->hostname, ip->protocol_state, strerror(errno));
 	  
-	  authentication_type = ip->authentication_type;
+	  authentication_type = cmd_args.common.authentication_type;
 	  if (authentication_type != IPMI_AUTHENTICATION_TYPE_NONE)
-	    {
-	      if (strlen(conf->password))
-		password = conf->password;
-	      else
-		password = NULL;
-	    }
+            password = cmd_args.common.password;
 	  else
 	    password = NULL;
 	  
@@ -185,38 +177,31 @@ ipmipower_check_authentication_code(ipmipower_powercmd_t ip,
 								      buffer_len,
 								      authentication_type,
 								      (uint8_t *)password,
-								      strlen(conf->password))) < 0)
-	    err_exit("ipmipower_check_authentication_code(%s:%d): "
-		     "ipmi_lan_check_session_authentication_code: %s",
-		     ip->ic->hostname, ip->protocol_state, strerror(errno));
+								      password ? strlen(password) : 0)) < 0)
+	    ierr_exit("ipmipower_check_authentication_code(%s:%d): "
+                      "ipmi_lan_check_session_authentication_code: %s",
+                      ip->ic->hostname, ip->protocol_state, strerror(errno));
 	  
 	  if (rv)
-	    dbg("ipmipower_check_authentication_code(%s:%d): "
-		"permsgauth authcode re-check passed",
-		ip->ic->hostname, ip->protocol_state);
+	    ierr_dbg("ipmipower_check_authentication_code(%s:%d): "
+                     "permsgauth authcode re-check passed",
+                     ip->ic->hostname, ip->protocol_state);
 	}
     }      
   else	/* 
-           (pkt == GET_CHANNEL_CIPHER_SUITES_RES 
-	    || (ip->ipmi_version == IPMI_VERSION_2_0
-	        && (pkt == SET_SESSION_PRIVILEGE_LEVEL_RES
-		    || pkt == GET_CHASSIS_STATUS_RES
-		    || pkt == CHASSIS_CONTROL_RES
-		    || pkt == CLOSE_SESSION_RES)))
+           (cmd_args.common.driver_type == IPMI_DEVICE_LAN_2_0
+           && (pkt == SET_SESSION_PRIVILEGE_LEVEL_RES
+           || pkt == GET_CHASSIS_STATUS_RES
+           || pkt == CHASSIS_CONTROL_RES
+           || pkt == CLOSE_SESSION_RES))
 	*/
     {
       /* IPMI 2.0 Checks */
       uint8_t integrity_algorithm;
 
-      if (pkt == GET_CHANNEL_CIPHER_SUITES_RES)
-	integrity_algorithm = IPMI_INTEGRITY_ALGORITHM_NONE;
-      else
-	integrity_algorithm = ip->integrity_algorithm;
+      integrity_algorithm = ip->integrity_algorithm;
 
-      if (strlen(conf->password))
-	password = conf->password;
-      else
-	password = NULL;
+      password = cmd_args.common.password;
 	  
       if ((rv = ipmi_rmcpplus_check_packet_session_authentication_code(integrity_algorithm,
 								       buffer,
@@ -226,15 +211,15 @@ ipmipower_check_authentication_code(ipmipower_powercmd_t ip,
 								       (uint8_t *)password,
 								       (password) ? strlen(password) : 0,
 								       ip->obj_rmcpplus_session_trlr_res)) < 0)
-	err_exit("ipmipower_check_authentication_code(%s:%d): "
-		 "ipmi_rmcpplus_check_session_authentication_code: %s",
-		 ip->ic->hostname, ip->protocol_state, strerror(errno));
+	ierr_exit("ipmipower_check_authentication_code(%s:%d): "
+                  "ipmi_rmcpplus_check_session_authentication_code: %s",
+                  ip->ic->hostname, ip->protocol_state, strerror(errno));
     }
 
   if (!rv)
-    dbg("ipmipower_check_authentication_code(%s:%d): "
-	"authentication code check failed",
-	ip->ic->hostname, ip->protocol_state);
+    ierr_dbg("ipmipower_check_authentication_code(%s:%d): "
+             "authentication code check failed",
+             ip->ic->hostname, ip->protocol_state);
   
   return (int)rv;
 }
@@ -246,7 +231,7 @@ ipmipower_check_outbound_sequence_number(ipmipower_powercmd_t ip, packet_type_t
   uint64_t seq_num = 0;
   int rv = 0;
 
-  assert(ip != NULL);
+  assert(ip);
   assert(PACKET_TYPE_VALID_RES(pkt));
   
   /* achu: This algorithm is more or less from Appendix A of the IPMI
@@ -265,13 +250,12 @@ ipmipower_check_outbound_sequence_number(ipmipower_powercmd_t ip, packet_type_t
   if (pkt == AUTHENTICATION_CAPABILITIES_V20_RES 
       || pkt == AUTHENTICATION_CAPABILITIES_RES 
       || pkt == GET_SESSION_CHALLENGE_RES
-      || pkt == GET_CHANNEL_CIPHER_SUITES_RES
       || pkt == OPEN_SESSION_RES
       || pkt == RAKP_MESSAGE_2_RES
       || pkt == RAKP_MESSAGE_4_RES)
     return 1;
 
-  if (ip->ipmi_version == IPMI_VERSION_2_0
+  if (cmd_args.common.driver_type == IPMI_DEVICE_LAN_2_0
       && (pkt == SET_SESSION_PRIVILEGE_LEVEL_RES
 	  || pkt == GET_CHASSIS_STATUS_RES
 	  || pkt == CHASSIS_CONTROL_RES
@@ -281,11 +265,11 @@ ipmipower_check_outbound_sequence_number(ipmipower_powercmd_t ip, packet_type_t
 		 &seq_num);
   else /* 
 	  pkt == ACTIVATE_SESSION_RES  
-	  || (ip->ipmi_version == IPMI_VERSION_1_5
-	      && (pkt == SET_SESSION_PRIVILEGE_LEVEL_RES
-	      || pkt == GET_CHASSIS_STATUS_RES
-	      || pkt == CHASSIS_CONTROL_RES
-	      || pkt == CLOSE_SESSION_RES))
+	  || (cmd_args.common.driver_type == IPMI_DEVICE_LAN
+          && (pkt == SET_SESSION_PRIVILEGE_LEVEL_RES
+          || pkt == GET_CHASSIS_STATUS_RES
+          || pkt == CHASSIS_CONTROL_RES
+          || pkt == CLOSE_SESSION_RES))
        */
     Fiid_obj_get(ip->obj_lan_session_hdr_res,
 		 "session_sequence_number", 
@@ -298,7 +282,7 @@ ipmipower_check_outbound_sequence_number(ipmipower_powercmd_t ip, packet_type_t
    * The session sequence numbers for IPMI 1.5 are the wrong endian.
    * So we have to flip the bits to workaround it.
    */
-  if (conf->workaround_flags & WORKAROUND_FLAG_BIG_ENDIAN_SEQUENCE_NUMBER)
+  if (cmd_args.common.workaround_flags & IPMI_TOOL_WORKAROUND_FLAGS_BIG_ENDIAN_SEQUENCE_NUMBER)
     {
       uint32_t tmp_seq_num = seq_num;
 
@@ -325,7 +309,7 @@ ipmipower_check_outbound_sequence_number(ipmipower_powercmd_t ip, packet_type_t
     goto out;
 
   /* In IPMI 2.0, sequence number 0 is special, and shouldn't happen */
-  if (ip->ipmi_version == IPMI_VERSION_2_0 && seq_num == 0)
+  if (cmd_args.common.driver_type == IPMI_DEVICE_LAN_2_0 && !seq_num)
     goto out;
 
   /* Check if sequence number is greater than highest received and is
@@ -336,7 +320,7 @@ ipmipower_check_outbound_sequence_number(ipmipower_powercmd_t ip, packet_type_t
       wrap_val = IPMIPOWER_SEQUENCE_NUMBER_WINDOW - (IPMIPOWER_MAX_SEQUENCE_NUMBER - ip->highest_received_sequence_number) - 1;
 
       /* In IPMI 2.0, sequence number 0 isn't possible, so adjust wrap_val */
-      if (ip->ipmi_version == IPMI_VERSION_2_0)
+      if (cmd_args.common.driver_type == IPMI_DEVICE_LAN_2_0)
 	wrap_val++;
 
       if (seq_num > ip->highest_received_sequence_number || seq_num <= wrap_val)
@@ -345,7 +329,7 @@ ipmipower_check_outbound_sequence_number(ipmipower_powercmd_t ip, packet_type_t
             shift_num = seq_num - ip->highest_received_sequence_number;
           else
             {
-              if (ip->ipmi_version == IPMI_VERSION_1_5)
+              if (cmd_args.common.driver_type == IPMI_DEVICE_LAN)
                 shift_num = seq_num + (IPMIPOWER_MAX_SEQUENCE_NUMBER - ip->highest_received_sequence_number) + 1;
               else
                 /* IPMI 2.0 Special Case b/c 0 isn't a legit sequence number */
@@ -379,14 +363,14 @@ ipmipower_check_outbound_sequence_number(ipmipower_powercmd_t ip, packet_type_t
       uint32_t wrap_val = IPMIPOWER_MAX_SEQUENCE_NUMBER - (IPMIPOWER_SEQUENCE_NUMBER_WINDOW - ip->highest_received_sequence_number) + 1;
       
       /* In IPMI 2.0, sequence number 0 isn't possible, so adjust wrap_val */
-      if (ip->ipmi_version == IPMI_VERSION_2_0)
+      if (cmd_args.common.driver_type == IPMI_DEVICE_LAN_2_0)
 	wrap_val--;
 
       if (seq_num < ip->highest_received_sequence_number || seq_num >= wrap_val)
         {
           if (seq_num > ip->highest_received_sequence_number && seq_num <= IPMIPOWER_MAX_SEQUENCE_NUMBER)
             {
-              if (ip->ipmi_version == IPMI_VERSION_1_5)
+              if (cmd_args.common.driver_type == IPMI_DEVICE_LAN)
                 shift_num = ip->highest_received_sequence_number + (IPMIPOWER_MAX_SEQUENCE_NUMBER - seq_num) + 1;
               else
                 /* IPMI 2.0 Special Case b/c 0 isn't a legit sequence number */
@@ -435,9 +419,9 @@ ipmipower_check_outbound_sequence_number(ipmipower_powercmd_t ip, packet_type_t
 
  out:
   if (!rv)
-    dbg("ipmipower_check_outbound_sequence_number(%s:%d): seq_num: %u, high: %u",
-        ip->ic->hostname, ip->protocol_state, (unsigned int)seq_num, 
-	ip->highest_received_sequence_number);
+    ierr_dbg("ipmipower_check_outbound_sequence_number(%s:%d): seq_num: %u, high: %u",
+             ip->ic->hostname, ip->protocol_state, (unsigned int)seq_num, 
+             ip->highest_received_sequence_number);
   
   return rv;
 }
@@ -448,16 +432,15 @@ ipmipower_check_session_id(ipmipower_powercmd_t ip, packet_type_t pkt)
   uint64_t session_id = 0;
   uint64_t expected_session_id = 0;
 
-  assert(ip != NULL);
+  assert(ip);
   assert(PACKET_TYPE_VALID_RES(pkt));
     
   if (pkt == AUTHENTICATION_CAPABILITIES_V20_RES 
       || pkt == AUTHENTICATION_CAPABILITIES_RES 
       || pkt == GET_SESSION_CHALLENGE_RES 
-      || pkt == ACTIVATE_SESSION_RES
-      || pkt == GET_CHANNEL_CIPHER_SUITES_RES)      
+      || pkt == ACTIVATE_SESSION_RES)
     return 1;
-  else if (ip->ipmi_version == IPMI_VERSION_1_5
+  else if (cmd_args.common.driver_type == IPMI_DEVICE_LAN
 	   && (pkt == SET_SESSION_PRIVILEGE_LEVEL_RES
 	       || pkt == GET_CHASSIS_STATUS_RES
 	       || pkt == CHASSIS_CONTROL_RES
@@ -470,7 +453,7 @@ ipmipower_check_session_id(ipmipower_powercmd_t ip, packet_type_t pkt)
                    "session_id", 
 		   &expected_session_id);
     }
-  else if (ip->ipmi_version == IPMI_VERSION_2_0
+  else if (cmd_args.common.driver_type == IPMI_DEVICE_LAN_2_0
 	   && (pkt == SET_SESSION_PRIVILEGE_LEVEL_RES
 	       || pkt == GET_CHASSIS_STATUS_RES
 	       || pkt == CHASSIS_CONTROL_RES
@@ -496,9 +479,9 @@ ipmipower_check_session_id(ipmipower_powercmd_t ip, packet_type_t pkt)
     }
   
   if (session_id != expected_session_id)
-    dbg("ipmipower_check_session_id(%s:%d): session id: %x expected: %x",
-        ip->ic->hostname, ip->protocol_state, (unsigned int)session_id, 
-        (unsigned int)expected_session_id);
+    ierr_dbg("ipmipower_check_session_id(%s:%d): session id: %x expected: %x",
+             ip->ic->hostname, ip->protocol_state, (unsigned int)session_id, 
+             (unsigned int)expected_session_id);
   
   /* IPMI Workaround (achu)
    *
@@ -509,7 +492,7 @@ ipmipower_check_session_id(ipmipower_powercmd_t ip, packet_type_t pkt)
    * session id is correct if it is equal to zero.
    */
 
-  if ((conf->workaround_flags & WORKAROUND_FLAG_ACCEPT_SESSION_ID_ZERO)
+  if ((cmd_args.common.workaround_flags & IPMI_TOOL_WORKAROUND_FLAGS_ACCEPT_SESSION_ID_ZERO)
       && !session_id)
     return (1);
 
@@ -522,7 +505,7 @@ ipmipower_check_network_function(ipmipower_powercmd_t ip, packet_type_t pkt)
   uint64_t netfn = 0;
   uint64_t expected_netfn;
 
-  assert(ip != NULL);
+  assert(ip);
   assert(PACKET_TYPE_VALID_RES(pkt));
   /* Assert this is not an IPMI 2.0 Session Setup Packet */
   assert(pkt != OPEN_SESSION_RES
@@ -537,8 +520,8 @@ ipmipower_check_network_function(ipmipower_powercmd_t ip, packet_type_t pkt)
     expected_netfn = IPMI_NET_FN_APP_RS;
   
   if (netfn != expected_netfn)
-    dbg("ipmipower_check_network_function(%s:%d): netfn: %x, expected: %x", 
-        ip->ic->hostname, ip->protocol_state, (unsigned int)netfn, expected_netfn);
+    ierr_dbg("ipmipower_check_network_function(%s:%d): netfn: %x, expected: %x", 
+             ip->ic->hostname, ip->protocol_state, (unsigned int)netfn, expected_netfn);
 
   return ((netfn == expected_netfn) ? 1 : 0);
 }
@@ -547,10 +530,10 @@ int
 ipmipower_check_command(ipmipower_powercmd_t ip, packet_type_t pkt) 
 {
   uint64_t cmd = 0;
-  uint64_t expected_cmd = -1;
+  uint64_t expected_cmd = 0;
   fiid_obj_t obj_cmd;
 
-  assert(ip != NULL);
+  assert(ip);
   assert(PACKET_TYPE_VALID_RES(pkt));
   /* Assert this is not an IPMI 2.0 Session Setup Packet */
   assert(pkt != OPEN_SESSION_RES
@@ -568,8 +551,6 @@ ipmipower_check_command(ipmipower_powercmd_t ip, packet_type_t pkt)
     expected_cmd = IPMI_CMD_GET_SESSION_CHALLENGE;
   else if (pkt == ACTIVATE_SESSION_RES) 
     expected_cmd = IPMI_CMD_ACTIVATE_SESSION;
-  else if (pkt == GET_CHANNEL_CIPHER_SUITES_RES)
-    expected_cmd = IPMI_CMD_GET_CHANNEL_CIPHER_SUITES;
   else if (pkt == SET_SESSION_PRIVILEGE_LEVEL_RES) 
     expected_cmd = IPMI_CMD_SET_SESSION_PRIVILEGE_LEVEL;
   else if (pkt == GET_CHASSIS_STATUS_RES) 
@@ -580,9 +561,9 @@ ipmipower_check_command(ipmipower_powercmd_t ip, packet_type_t pkt)
     expected_cmd = IPMI_CMD_CLOSE_SESSION;
   
   if (cmd != expected_cmd)
-    dbg("ipmipower_check_command(%s:%d): cmd: %x, expected: %x", 
-        ip->ic->hostname, ip->protocol_state, 
-	(unsigned int)cmd, (unsigned int)expected_cmd);
+    ierr_dbg("ipmipower_check_command(%s:%d): cmd: %x, expected: %x", 
+             ip->ic->hostname, ip->protocol_state, 
+             (unsigned int)cmd, (unsigned int)expected_cmd);
   
   return ((cmd == expected_cmd) ? 1 : 0);
 }
@@ -593,7 +574,7 @@ ipmipower_check_requester_sequence_number(ipmipower_powercmd_t ip, packet_type_t
   uint64_t req_seq = 0;
   uint64_t expected_req_seq = 0;
 
-  assert(ip != NULL);
+  assert(ip);
   assert(PACKET_TYPE_VALID_RES(pkt));
   /* Assert this is not an IPMI 2.0 Session Setup Packet */
   assert(pkt != OPEN_SESSION_RES
@@ -605,9 +586,9 @@ ipmipower_check_requester_sequence_number(ipmipower_powercmd_t ip, packet_type_t
   Fiid_obj_get(ip->obj_lan_msg_hdr_res, "rq_seq", &req_seq);
 
   if (req_seq != expected_req_seq)
-    dbg("ipmipower_check_requester_sequence_number(%s:%d): req_seq: %x, expected: %x",
-        ip->ic->hostname, ip->protocol_state, 
-	(unsigned int)req_seq, (unsigned int)expected_req_seq);
+    ierr_dbg("ipmipower_check_requester_sequence_number(%s:%d): req_seq: %x, expected: %x",
+             ip->ic->hostname, ip->protocol_state, 
+             (unsigned int)req_seq, (unsigned int)expected_req_seq);
   
   return ((req_seq == expected_req_seq) ? 1 : 0);
 }
@@ -618,7 +599,7 @@ ipmipower_check_completion_code(ipmipower_powercmd_t ip, packet_type_t pkt)
   uint64_t comp_code = 0;
   fiid_obj_t obj_cmd;
   
-  assert(ip != NULL);
+  assert(ip);
   assert(PACKET_TYPE_VALID_RES(pkt));
   /* Assert this is not an IPMI 2.0 Session Setup Packet */
   assert(pkt != OPEN_SESSION_RES
@@ -630,8 +611,8 @@ ipmipower_check_completion_code(ipmipower_powercmd_t ip, packet_type_t pkt)
   Fiid_obj_get(obj_cmd, "comp_code", &comp_code);
   
   if (comp_code != IPMI_COMP_CODE_COMMAND_SUCCESS)
-    dbg("ipmipower_check_completion_code(%s:%d): comp_code: %x", 
-        ip->ic->hostname, ip->protocol_state, (unsigned int)comp_code);
+    ierr_dbg("ipmipower_check_completion_code(%s:%d): comp_code: %x", 
+             ip->ic->hostname, ip->protocol_state, (unsigned int)comp_code);
   
   return ((comp_code == IPMI_COMP_CODE_COMMAND_SUCCESS) ? 1 : 0);
 }
@@ -642,13 +623,12 @@ ipmipower_check_payload_type(ipmipower_powercmd_t ip, packet_type_t pkt)
   uint64_t payload_type;
   uint8_t expected_payload_type;
 
-  assert(ip != NULL);
+  assert(ip);
   assert(PACKET_TYPE_VALID_RES(pkt));
-  assert(pkt == GET_CHANNEL_CIPHER_SUITES_RES
-         || pkt == OPEN_SESSION_RES
+  assert(pkt == OPEN_SESSION_RES
 	 || pkt == RAKP_MESSAGE_2_RES
 	 || pkt == RAKP_MESSAGE_4_RES
-	 || (ip->ipmi_version == IPMI_VERSION_2_0
+	 || (cmd_args.common.driver_type == IPMI_DEVICE_LAN_2_0
 	     && (pkt == SET_SESSION_PRIVILEGE_LEVEL_RES
 		 || pkt == GET_CHASSIS_STATUS_RES
 		 || pkt == CHASSIS_CONTROL_RES
@@ -668,10 +648,10 @@ ipmipower_check_payload_type(ipmipower_powercmd_t ip, packet_type_t pkt)
     expected_payload_type = IPMI_PAYLOAD_TYPE_IPMI;
 
   if (payload_type != expected_payload_type)
-    dbg("ipmipower_check_payload_type(%s:%d): "
-	"payload_type: %x, expected: %x",
-        ip->ic->hostname, ip->protocol_state, 
-	(unsigned int)payload_type, (unsigned int)expected_payload_type);
+    ierr_dbg("ipmipower_check_payload_type(%s:%d): "
+             "payload_type: %x, expected: %x",
+             ip->ic->hostname, ip->protocol_state, 
+             (unsigned int)payload_type, (unsigned int)expected_payload_type);
 
   return ((payload_type == expected_payload_type) ? 1 : 0);
 }
@@ -683,7 +663,7 @@ ipmipower_check_message_tag(ipmipower_powercmd_t ip, packet_type_t pkt)
   uint8_t expected_message_tag;
   fiid_obj_t obj_cmd;
 
-  assert(ip != NULL);
+  assert(ip);
   assert(PACKET_TYPE_VALID_RES(pkt));
   assert(pkt == OPEN_SESSION_RES
 	 || pkt == RAKP_MESSAGE_2_RES
@@ -698,11 +678,11 @@ ipmipower_check_message_tag(ipmipower_powercmd_t ip, packet_type_t pkt)
   expected_message_tag = ip->initial_message_tag + ip->message_tag_count;
 
   if (message_tag != expected_message_tag)
-    dbg("ipmipower_check_message_tag(%s:%d): "
-	"message_tag: %x, expected: %x",
-        ip->ic->hostname, ip->protocol_state, 
-	(unsigned int)message_tag, 
-	(unsigned int)expected_message_tag);
+    ierr_dbg("ipmipower_check_message_tag(%s:%d): "
+             "message_tag: %x, expected: %x",
+             ip->ic->hostname, ip->protocol_state, 
+             (unsigned int)message_tag, 
+             (unsigned int)expected_message_tag);
 
   return ((message_tag == expected_message_tag) ? 1 : 0);
 }
@@ -713,7 +693,7 @@ ipmipower_check_rmcpplus_status_code(ipmipower_powercmd_t ip, packet_type_t pkt)
   uint64_t rmcpplus_status_code;
   fiid_obj_t obj_cmd;
 
-  assert(ip != NULL);
+  assert(ip);
   assert(PACKET_TYPE_VALID_RES(pkt));
   assert(pkt == OPEN_SESSION_RES
 	 || pkt == RAKP_MESSAGE_2_RES
@@ -726,10 +706,10 @@ ipmipower_check_rmcpplus_status_code(ipmipower_powercmd_t ip, packet_type_t pkt)
 	       &rmcpplus_status_code);
 
   if (rmcpplus_status_code != RMCPPLUS_STATUS_NO_ERRORS)
-    dbg("ipmipower_check_rmcpplus_status_code(%s:%d): "
-	"rmcpplus_status_code: %x",
-        ip->ic->hostname, ip->protocol_state, 
-	(unsigned int)rmcpplus_status_code);
+    ierr_dbg("ipmipower_check_rmcpplus_status_code(%s:%d): "
+             "rmcpplus_status_code: %x",
+             ip->ic->hostname, ip->protocol_state, 
+             (unsigned int)rmcpplus_status_code);
 
   return ((rmcpplus_status_code == RMCPPLUS_STATUS_NO_ERRORS) ? 1 : 0);
 }
@@ -740,7 +720,7 @@ ipmipower_check_open_session_response_privilege(ipmipower_powercmd_t ip, packet_
   uint64_t val;
   int rv;
 
-  assert(ip != NULL);
+  assert(ip);
   assert(pkt == OPEN_SESSION_RES);
   
   /*
@@ -761,22 +741,22 @@ ipmipower_check_open_session_response_privilege(ipmipower_powercmd_t ip, packet_
    * The Intel's don't work with IPMI_PRIVILEGE_LEVEL_HIGHEST_LEVEL.
    * So check that we get back what we sent.
    */
-  if (conf->workaround_flags & WORKAROUND_FLAG_INTEL_2_0_SESSION)
+  if (cmd_args.common.workaround_flags & IPMI_TOOL_WORKAROUND_FLAGS_INTEL_2_0_SESSION)
     rv = (val == ip->requested_maximum_privilege_level) ? 1 : 0;
   else
     {
-      if (ip->privilege_level == IPMI_PRIVILEGE_LEVEL_USER
+      if (cmd_args.common.privilege_level == IPMI_PRIVILEGE_LEVEL_USER
 	  && (val == IPMI_PRIVILEGE_LEVEL_USER
 	      || val == IPMI_PRIVILEGE_LEVEL_OPERATOR
 	      || val == IPMI_PRIVILEGE_LEVEL_ADMIN
 	      || val == IPMI_PRIVILEGE_LEVEL_OEM))
 	rv = 1;
-      else if (ip->privilege_level == IPMI_PRIVILEGE_LEVEL_OPERATOR
+      else if (cmd_args.common.privilege_level == IPMI_PRIVILEGE_LEVEL_OPERATOR
 	       && (val == IPMI_PRIVILEGE_LEVEL_OPERATOR
 		   || val == IPMI_PRIVILEGE_LEVEL_ADMIN
 		   || val == IPMI_PRIVILEGE_LEVEL_OEM))
-     rv = 1;
-      else if (ip->privilege_level == IPMI_PRIVILEGE_LEVEL_ADMIN
+        rv = 1;
+      else if (cmd_args.common.privilege_level == IPMI_PRIVILEGE_LEVEL_ADMIN
 	       && (val == IPMI_PRIVILEGE_LEVEL_ADMIN
 		   || val == IPMI_PRIVILEGE_LEVEL_OEM))
 	rv = 1;
@@ -785,11 +765,11 @@ ipmipower_check_open_session_response_privilege(ipmipower_powercmd_t ip, packet_
     }
 
   if (!rv)
-    dbg("ipmipower_check_open_session_response_privilege(%s:%d): "
-        "invalid privilege: %x, expected: %x",
-        ip->ic->hostname, ip->protocol_state,
-        (unsigned int)val,
-        (unsigned int)ip->requested_maximum_privilege_level);
+    ierr_dbg("ipmipower_check_open_session_response_privilege(%s:%d): "
+             "invalid privilege: %x, expected: %x",
+             ip->ic->hostname, ip->protocol_state,
+             (unsigned int)val,
+             (unsigned int)ip->requested_maximum_privilege_level);
   
   return (rv);
 }
@@ -808,7 +788,7 @@ ipmipower_check_rakp_2_key_exchange_authentication_code(ipmipower_powercmd_t ip,
   uint64_t managed_system_session_id;
   int8_t rv;
 
-  assert(ip != NULL);
+  assert(ip);
   assert(pkt == RAKP_MESSAGE_2_RES);
 
   /* IPMI Workaround (achu)
@@ -821,26 +801,23 @@ ipmipower_check_rakp_2_key_exchange_authentication_code(ipmipower_powercmd_t ip,
    * Table 13-11 in the IPMI 2.0 spec.
    */
 
-  if (conf->workaround_flags & WORKAROUND_FLAG_INTEL_2_0_SESSION)
+  if (cmd_args.common.workaround_flags & IPMI_TOOL_WORKAROUND_FLAGS_INTEL_2_0_SESSION)
     {
       memset(username_buf, '\0', IPMI_MAX_USER_NAME_LENGTH+1);
-      if (strlen(conf->username))
-	strcpy(username_buf, conf->username);
+      if (cmd_args.common.username)
+	strcpy(username_buf, cmd_args.common.username);
       username = username_buf;
       username_len = IPMI_MAX_USER_NAME_LENGTH;
     }
   else
     {
-      if (strlen(conf->username))
-	username = conf->username;
-      else
-	username = NULL;
+      username = cmd_args.common.username;
       username_len = (username) ? strlen(username) : 0;
     }
   
-  if (conf->workaround_flags & WORKAROUND_FLAG_SUPERMICRO_2_0_SESSION)
+  if (cmd_args.common.workaround_flags & IPMI_TOOL_WORKAROUND_FLAGS_SUPERMICRO_2_0_SESSION)
     {
-      uint8_t keybuf[IPMI_PACKET_BUFLEN];
+      uint8_t keybuf[IPMIPOWER_PACKET_BUFLEN];
       int32_t keybuf_len;
 
       /* IPMI Workaround (achu) 
@@ -857,11 +834,11 @@ ipmipower_check_rakp_2_key_exchange_authentication_code(ipmipower_powercmd_t ip,
       keybuf_len = Fiid_obj_get_data(ip->obj_rakp_message_2_res,
 				     "key_exchange_authentication_code",
 				     keybuf,
-				     IPMI_PACKET_BUFLEN);
+				     IPMIPOWER_PACKET_BUFLEN);
       if (ip->authentication_algorithm == IPMI_AUTHENTICATION_ALGORITHM_RAKP_NONE
 	  && keybuf_len == 1) 
 	Fiid_obj_clear_field(ip->obj_rakp_message_2_res, "key_exchange_authentication_code");
-       else if (ip->authentication_algorithm == IPMI_AUTHENTICATION_ALGORITHM_RAKP_HMAC_SHA1
+      else if (ip->authentication_algorithm == IPMI_AUTHENTICATION_ALGORITHM_RAKP_HMAC_SHA1
 	       && keybuf_len == (IPMI_HMAC_SHA1_DIGEST_LENGTH + 1))
 	Fiid_obj_set_data(ip->obj_rakp_message_2_res,
 			  "key_exchange_authentication_code",
@@ -875,11 +852,7 @@ ipmipower_check_rakp_2_key_exchange_authentication_code(ipmipower_powercmd_t ip,
 			  IPMI_HMAC_MD5_DIGEST_LENGTH);
     }
 
-  if (strlen(conf->password))
-    password = conf->password;
-  else
-    password = NULL;
-
+  password = cmd_args.common.password;
   password_len = (password) ? strlen(password) : 0;
 
   /* IPMI Workaround (achu)
@@ -892,7 +865,7 @@ ipmipower_check_rakp_2_key_exchange_authentication_code(ipmipower_powercmd_t ip,
    * password to 16 bytes when generating keys, hashes, etc.  So we
    * have to do the same when generating keys, hashes, etc.
    */
-  if (conf->workaround_flags & WORKAROUND_FLAG_INTEL_2_0_SESSION 
+  if (cmd_args.common.workaround_flags & IPMI_TOOL_WORKAROUND_FLAGS_INTEL_2_0_SESSION 
       && ip->authentication_algorithm == IPMI_AUTHENTICATION_ALGORITHM_RAKP_HMAC_MD5
       && password_len > IPMI_1_5_MAX_PASSWORD_LENGTH)
     password_len = IPMI_1_5_MAX_PASSWORD_LENGTH;
@@ -921,7 +894,7 @@ ipmipower_check_rakp_2_key_exchange_authentication_code(ipmipower_powercmd_t ip,
    * Notes: Cipher suite 1,2,3 are the ones that use HMAC-SHA1 and
    * have the problem.
    */
-  if (conf->workaround_flags & WORKAROUND_FLAG_SUN_2_0_SESSION
+  if (cmd_args.common.workaround_flags & IPMI_TOOL_WORKAROUND_FLAGS_SUN_2_0_SESSION
       && (ip->authentication_algorithm == IPMI_AUTHENTICATION_ALGORITHM_RAKP_HMAC_SHA1))
     {
       uint8_t buf[IPMI_MAX_KEY_EXCHANGE_AUTHENTICATION_CODE_LENGTH];
@@ -955,18 +928,18 @@ ipmipower_check_rakp_2_key_exchange_authentication_code(ipmipower_powercmd_t ip,
                                                                         managed_system_guid,
                                                                         managed_system_guid_len,
                                                                         ip->name_only_lookup,
-                                                                        ip->privilege_level,
+                                                                        cmd_args.common.privilege_level,
                                                                         username,
                                                                         username_len,
                                                                         ip->obj_rakp_message_2_res)) < 0)
-    err_exit("ipmipower_check_rakp_2_key_exchange_authentication_code(%s:%d): "
-	     "ipmi_rmcpplus_check_rakp_message_2_key_exchange_authentication_code: %s",
-	     ip->ic->hostname, ip->protocol_state, strerror(errno));
+    ierr_exit("ipmipower_check_rakp_2_key_exchange_authentication_code(%s:%d): "
+              "ipmi_rmcpplus_check_rakp_message_2_key_exchange_authentication_code: %s",
+              ip->ic->hostname, ip->protocol_state, strerror(errno));
 
   if (!rv)
-    dbg("ipmipower_check_rakp_2_key_exchange_authentication_code(%s:%d): "
-	"rakp 2 check failed",
-        ip->ic->hostname, ip->protocol_state);
+    ierr_dbg("ipmipower_check_rakp_2_key_exchange_authentication_code(%s:%d): "
+             "rakp 2 check failed",
+             ip->ic->hostname, ip->protocol_state);
 
   return (int)rv;
 }
@@ -981,7 +954,7 @@ ipmipower_check_rakp_4_integrity_check_value(ipmipower_powercmd_t ip, packet_typ
   uint64_t val;
   int8_t rv;
 
-  assert(ip != NULL);
+  assert(ip);
   assert(pkt == RAKP_MESSAGE_4_RES);
 
   /* IPMI Workaround (achu)
@@ -997,7 +970,7 @@ ipmipower_check_rakp_4_integrity_check_value(ipmipower_powercmd_t ip, packet_typ
    * one.  Would have taken me awhile to figure this one out :-)
    */
 
-  if (conf->workaround_flags & WORKAROUND_FLAG_INTEL_2_0_SESSION)
+  if (cmd_args.common.workaround_flags & IPMI_TOOL_WORKAROUND_FLAGS_INTEL_2_0_SESSION)
     {
       if (ip->integrity_algorithm == IPMI_INTEGRITY_ALGORITHM_NONE)
         authentication_algorithm = IPMI_AUTHENTICATION_ALGORITHM_RAKP_NONE;
@@ -1035,14 +1008,14 @@ ipmipower_check_rakp_4_integrity_check_value(ipmipower_powercmd_t ip, packet_typ
                                                              managed_system_guid,
                                                              managed_system_guid_len,
                                                              ip->obj_rakp_message_4_res)) < 0)
-    err_exit("ipmipower_check_rakp_4_integrity_check_value(%s:%d): "
-	     "ipmipower_check_rakp_4_integrity_check_value: %s",
-	     ip->ic->hostname, ip->protocol_state, strerror(errno));
+    ierr_exit("ipmipower_check_rakp_4_integrity_check_value(%s:%d): "
+              "ipmipower_check_rakp_4_integrity_check_value: %s",
+              ip->ic->hostname, ip->protocol_state, strerror(errno));
 
   if (!rv)
-    dbg("ipmipower_check_rakp_4_integrity_check_value(%s:%d): "
-	"rakp 4 check failed",
-        ip->ic->hostname, ip->protocol_state);
+    ierr_dbg("ipmipower_check_rakp_4_integrity_check_value(%s:%d): "
+             "rakp 4 check failed",
+             ip->ic->hostname, ip->protocol_state);
 
   return (int)rv;
 }
@@ -1053,30 +1026,26 @@ ipmipower_check_payload_pad(ipmipower_powercmd_t ip, packet_type_t pkt)
   uint8_t confidentiality_algorithm;
   int8_t rv;
 
-  assert(ip != NULL);
+  assert(ip);
   assert(PACKET_TYPE_VALID_RES(pkt));
-  assert(pkt == GET_CHANNEL_CIPHER_SUITES_RES
-         || (ip->ipmi_version == IPMI_VERSION_2_0
-             && (pkt == SET_SESSION_PRIVILEGE_LEVEL_RES
-                 || pkt == GET_CHASSIS_STATUS_RES
-                 || pkt == CHASSIS_CONTROL_RES
-                 || pkt == CLOSE_SESSION_RES)));
+  assert(cmd_args.common.driver_type == IPMI_DEVICE_LAN_2_0
+         && (pkt == SET_SESSION_PRIVILEGE_LEVEL_RES
+             || pkt == GET_CHASSIS_STATUS_RES
+             || pkt == CHASSIS_CONTROL_RES
+             || pkt == CLOSE_SESSION_RES));
 
-  if (pkt == GET_CHANNEL_CIPHER_SUITES_RES)
-    confidentiality_algorithm = IPMI_CONFIDENTIALITY_ALGORITHM_NONE;
-  else
-    confidentiality_algorithm = ip->confidentiality_algorithm;
+  confidentiality_algorithm = ip->confidentiality_algorithm;
 
   if ((rv = ipmi_rmcpplus_check_payload_pad(confidentiality_algorithm,
 					    ip->obj_rmcpplus_payload_res)) < 0)
-    err_exit("ipmipower_check_payload_pad(%s:%d): "
-	     "ipmi_rmcpplus_check_payload_pad: %s",
-	     ip->ic->hostname, ip->protocol_state, strerror(errno));
+    ierr_exit("ipmipower_check_payload_pad(%s:%d): "
+              "ipmi_rmcpplus_check_payload_pad: %s",
+              ip->ic->hostname, ip->protocol_state, strerror(errno));
 
   if (!rv)
-    dbg("ipmipower_check_payload_pad(%s:%d): "
-	"payload pad check failed",
-        ip->ic->hostname, ip->protocol_state);
+    ierr_dbg("ipmipower_check_payload_pad(%s:%d): "
+             "payload pad check failed",
+             ip->ic->hostname, ip->protocol_state);
 
   return (int)rv;
 }
@@ -1086,24 +1055,26 @@ ipmipower_check_integrity_pad(ipmipower_powercmd_t ip, packet_type_t pkt)
 {
   int8_t rv;
 
-  assert(ip != NULL);
+  assert(ip);
   assert(PACKET_TYPE_VALID_RES(pkt));
-  assert(pkt == GET_CHANNEL_CIPHER_SUITES_RES
-         || (ip->ipmi_version == IPMI_VERSION_2_0
-             && (pkt == SET_SESSION_PRIVILEGE_LEVEL_RES
-                 || pkt == GET_CHASSIS_STATUS_RES
-                 || pkt == CHASSIS_CONTROL_RES
-                 || pkt == CLOSE_SESSION_RES)));
+  assert(cmd_args.common.driver_type == IPMI_DEVICE_LAN_2_0
+         && (pkt == SET_SESSION_PRIVILEGE_LEVEL_RES
+             || pkt == GET_CHASSIS_STATUS_RES
+             || pkt == CHASSIS_CONTROL_RES
+             || pkt == CLOSE_SESSION_RES));
+
+  if (ip->integrity_algorithm == IPMI_INTEGRITY_ALGORITHM_NONE)
+    return 1;
 
   if ((rv = ipmi_rmcpplus_check_integrity_pad(ip->obj_rmcpplus_session_trlr_res)) < 0)
-    err_exit("ipmipower_check_integrity_pad(%s:%d): "
-	     "ipmi_rmcpplus_check_integrity_pad: %s",
-	     ip->ic->hostname, ip->protocol_state, strerror(errno));
+    ierr_exit("ipmipower_check_integrity_pad(%s:%d): "
+              "ipmi_rmcpplus_check_integrity_pad: %s",
+              ip->ic->hostname, ip->protocol_state, strerror(errno));
 
   if (!rv)
-    dbg("ipmipower_check_integrity_pad(%s:%d): "
-	"integrity pad check failed",
-        ip->ic->hostname, ip->protocol_state);
+    ierr_dbg("ipmipower_check_integrity_pad(%s:%d): "
+             "integrity pad check failed",
+             ip->ic->hostname, ip->protocol_state);
 
   return (int)rv;
 }
