@@ -1,7 +1,7 @@
 /*****************************************************************************\
- *  $Id: ipmipower_output.c,v 1.26 2007/10/18 16:18:52 chu11 Exp $
+ *  $Id: ipmipower_output.c,v 1.45 2008/08/12 18:14:41 chu11 Exp $
  *****************************************************************************
- *  Copyright (C) 2007 Lawrence Livermore National Security, LLC.
+ *  Copyright (C) 2007-2008 Lawrence Livermore National Security, LLC.
  *  Copyright (C) 2003-2007 The Regents of the University of California.
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
  *  Written by Albert Chu <chu11@llnl.gov>
@@ -39,50 +39,48 @@
 #include "ipmipower_output.h"
 #include "ipmipower_wrappers.h"
 
+#include "freeipmi-portability.h"
+
 extern cbuf_t ttyout;
-extern struct ipmipower_config *conf;
+extern struct ipmipower_arguments cmd_args;
 extern hostlist_t output_hostrange[MSG_TYPE_NUM_ENTRIES];
 
 static char *ipmipower_outputs[] = 
   {
-    "success",
     "on",
     "off",
     "ok",
-    "permission denied",
     "username invalid",
     "password invalid",
     "password length invalid",
     "k_g invalid",
-    "necessary privilege level for power control operation cannot be obtained",
     "privilege level cannot be obtained for this user",
     "operation invalid",
     "authentication type unavailable for attempted privilege level",
-    "usable authentication type/privilege combination not found",
     "cipher suite id unavailable",
-    "usable cipher suite id not found",
     "password verification timeout",
+    "connection timeout",
     "session timeout",
     "not discovered",
     "bad connection",
-    "unknown node name",
+    "invalid hostname",
+    "unconfigured hostname",
     "out of resources",
-    "ipmi 1.5 unavailable",
     "ipmi 2.0 unavailable",
     "BMC busy",
     "BMC error"
   };
 
 void
-ipmipower_output(msg_type_t num, char *hostname) 
+ipmipower_output(msg_type_t num, const char *hostname) 
 {
   assert(MSG_TYPE_VALID(num));
-  assert(hostname != NULL);
+  assert(hostname);
 
-  if (conf->consolidate_output == IPMIPOWER_TRUE)
+  if (cmd_args.hostrange.consolidate_output)
     {
-      if (hostlist_push_host(output_hostrange[num], hostname) == 0)
-        err_exit("hostlist_push_host() error\n");
+      if (!hostlist_push_host(output_hostrange[num], hostname))
+        ierr_exit("hostlist_push_host() error");
     }
   else
     cbuf_printf(ttyout, "%s: %s\n", hostname, ipmipower_outputs[num]);
@@ -93,20 +91,20 @@ ipmipower_output(msg_type_t num, char *hostname)
 void
 ipmipower_output_finish(void)
 {
-  if (conf->consolidate_output == IPMIPOWER_TRUE)
+  if (cmd_args.hostrange.consolidate_output)
     {
       int i, rv;
-      char buffer[IPMIPOWER_HOSTLIST_BUFLEN];
+      char buffer[IPMIPOWER_OUTPUT_BUFLEN];
       
       for (i = 0; i < MSG_TYPE_NUM_ENTRIES; i++) 
         {
           if (hostlist_count(output_hostrange[i]) > 0) {
-            memset(buffer, '\0', IPMIPOWER_HOSTLIST_BUFLEN); 
+            memset(buffer, '\0', IPMIPOWER_OUTPUT_BUFLEN); 
 
             hostlist_sort(output_hostrange[i]);
                 
             rv = hostlist_ranged_string(output_hostrange[i], 
-                                        IPMIPOWER_HOSTLIST_BUFLEN, 
+                                        IPMIPOWER_OUTPUT_BUFLEN, 
                                         buffer);
             if (rv < 0) 
               {
@@ -127,7 +125,7 @@ ipmipower_output_finish(void)
                 hostlist_delete(output_hostrange[i], buffer);
               }
             
-            assert(hostlist_count(output_hostrange[i]) == 0);
+            assert(!hostlist_count(output_hostrange[i]));
           }
         } 
     }
