@@ -1,6 +1,24 @@
+/*
+  Copyright (C) 2007-2008 FreeIPMI Core Team
+
+  This program is free software; you can redistribute it and/or modify
+  it under the terms of the GNU General Public License as published by
+  the Free Software Foundation; either version 2, or (at your option)
+  any later version.
+  
+  This program is distributed in the hope that it will be useful, but
+  WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+  General Public License for more details.
+
+  You should have received a copy of the GNU General Public License
+  along with this program; if not, write to the Free Software
+  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA
+*/
+
 #if HAVE_CONFIG_H
 #include "config.h"
-#endif
+#endif /* HAVE_CONFIG_H */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -14,6 +32,10 @@
 #include "pef-config-utils.h"
 #include "pef-config-validate.h"
 
+#include "freeipmi-portability.h"
+#include "pstdout.h"
+#include "tool-fiid-wrappers.h"
+
 /* achu: presumably there is no maximum.  We could read/write blocks
    forever based on block numbers.  However, we need to have some
    artificial max for the sake of pef-config.
@@ -41,34 +63,33 @@ _get_alert_string_keys (pef_config_state_data_t *state_data,
 
   string_selector = atoi (section_name + strlen ("Alert_String_"));
 
-  if (!(obj_cmd_rs = Fiid_obj_create(tmpl_cmd_get_pef_configuration_parameters_alert_string_keys_rs)))
-    goto cleanup;
+  _FIID_OBJ_CREATE(obj_cmd_rs, tmpl_cmd_get_pef_configuration_parameters_alert_string_keys_rs);
 
-  if (ipmi_cmd_get_pef_configuration_parameters_alert_string_keys (state_data->dev,
+  if (ipmi_cmd_get_pef_configuration_parameters_alert_string_keys (state_data->ipmi_ctx,
                                                                    IPMI_GET_PEF_PARAMETER,
                                                                    string_selector,
                                                                    BLOCK_SELECTOR,
                                                                    obj_cmd_rs) < 0)
     {
-      if (state_data->prog_data->args->common.flags & IPMI_FLAGS_DEBUG_DUMP)
-        fprintf(stderr,
-                "ipmi_cmd_get_pef_configuration_parameters_alert_string_keys: %s\n",
-                ipmi_device_strerror(ipmi_device_errnum(state_data->dev)));
-      rv = CONFIG_ERR_NON_FATAL_ERROR;
+      if (state_data->prog_data->args->config_args.common.debug)
+        pstdout_fprintf(state_data->pstate,
+                        stderr,
+                        "ipmi_cmd_get_pef_configuration_parameters_alert_string_keys: %s\n",
+                        ipmi_ctx_strerror(ipmi_ctx_errnum(state_data->ipmi_ctx)));
+      if (!IPMI_CTX_ERRNUM_IS_FATAL_ERROR(state_data->ipmi_ctx))
+        rv = CONFIG_ERR_NON_FATAL_ERROR;
       goto cleanup;
     }
 
-  if (Fiid_obj_get (obj_cmd_rs, "filter_number", &val) < 0)
-    goto cleanup;
+  _FIID_OBJ_GET (obj_cmd_rs, "filter_number", &val);
   ask->event_filter_number = val;
 
-  if (Fiid_obj_get (obj_cmd_rs, "set_number_for_string", &val) < 0)
-    goto cleanup;
+  _FIID_OBJ_GET (obj_cmd_rs, "set_number_for_string", &val);
   ask->alert_string_set = val;
 
   rv = CONFIG_ERR_SUCCESS;
  cleanup:
-  Fiid_obj_destroy(obj_cmd_rs);
+  _FIID_OBJ_DESTROY(obj_cmd_rs);
   return (rv);
 }
 
@@ -87,26 +108,27 @@ _set_alert_string_keys (pef_config_state_data_t *state_data,
 
   string_selector = atoi (section_name + strlen ("Alert_String_"));
 
-  if (!(obj_cmd_rs = Fiid_obj_create(tmpl_cmd_set_pef_configuration_parameters_rs)))
-    goto cleanup;
+  _FIID_OBJ_CREATE(obj_cmd_rs, tmpl_cmd_set_pef_configuration_parameters_rs);
 
-  if (ipmi_cmd_set_pef_configuration_parameters_alert_string_keys (state_data->dev,
+  if (ipmi_cmd_set_pef_configuration_parameters_alert_string_keys (state_data->ipmi_ctx,
                                                                    string_selector,
                                                                    ask->event_filter_number,
                                                                    ask->alert_string_set,
                                                                    obj_cmd_rs) < 0)
     {
-      if (state_data->prog_data->args->common.flags & IPMI_FLAGS_DEBUG_DUMP)
-        fprintf(stderr,
-                "ipmi_cmd_set_pef_configuration_parameters_alert_string_keys: %s\n",
-                ipmi_device_strerror(ipmi_device_errnum(state_data->dev)));
-      rv = CONFIG_ERR_NON_FATAL_ERROR;
+      if (state_data->prog_data->args->config_args.common.debug)
+        pstdout_fprintf(state_data->pstate,
+                        stderr,
+                        "ipmi_cmd_set_pef_configuration_parameters_alert_string_keys: %s\n",
+                        ipmi_ctx_strerror(ipmi_ctx_errnum(state_data->ipmi_ctx)));
+      if (!IPMI_CTX_ERRNUM_IS_FATAL_ERROR(state_data->ipmi_ctx))
+        rv = CONFIG_ERR_NON_FATAL_ERROR;
       goto cleanup;
     }
 
   rv = CONFIG_ERR_SUCCESS;
  cleanup:
-  Fiid_obj_destroy(obj_cmd_rs);
+  _FIID_OBJ_DESTROY(obj_cmd_rs);
   return (rv);
 }
 
@@ -124,7 +146,9 @@ event_filter_number_checkout (const char *section_name,
                                      &ask)) != CONFIG_ERR_SUCCESS)
     return ret;
 
-  if (config_section_update_keyvalue_output_int(kv, ask.event_filter_number) < 0)
+  if (config_section_update_keyvalue_output_int(state_data->pstate, 
+                                                kv, 
+                                                ask.event_filter_number) < 0)
     return CONFIG_ERR_FATAL_ERROR;
 
   return CONFIG_ERR_SUCCESS;
@@ -165,7 +189,9 @@ alert_string_set_checkout (const char *section_name,
                                      &ask)) != CONFIG_ERR_SUCCESS)
     return ret;
 
-  if (config_section_update_keyvalue_output_int(kv, ask.alert_string_set) < 0)
+  if (config_section_update_keyvalue_output_int(state_data->pstate, 
+                                                kv,
+                                                ask.alert_string_set) < 0)
     return CONFIG_ERR_FATAL_ERROR;
 
   return CONFIG_ERR_SUCCESS;
@@ -209,8 +235,7 @@ alert_string_checkout (const char *section_name,
 
   memset(alert_string, '\0', PEF_ALERT_STRING_MAX_LEN+1);
 
-  if (!(obj_cmd_rs = Fiid_obj_create(tmpl_cmd_get_pef_configuration_parameters_alert_strings_rs)))
-    goto cleanup;
+  _FIID_OBJ_CREATE(obj_cmd_rs, tmpl_cmd_get_pef_configuration_parameters_alert_strings_rs);
 
   if (!((PEF_ALERT_STRING_MAX_LEN) % 16))
     blocks = (PEF_ALERT_STRING_MAX_LEN)/16;
@@ -219,20 +244,23 @@ alert_string_checkout (const char *section_name,
 
   for (i = 0; i < blocks; i++)
     {
-      Fiid_obj_clear(obj_cmd_rs);
       int j;
 
-      if (ipmi_cmd_get_pef_configuration_parameters_alert_string (state_data->dev,
+      _FIID_OBJ_CLEAR(obj_cmd_rs);
+
+      if (ipmi_cmd_get_pef_configuration_parameters_alert_string (state_data->ipmi_ctx,
                                                                   IPMI_GET_PEF_PARAMETER,
                                                                   string_selector,
                                                                   i + 1,
                                                                   obj_cmd_rs) < 0)
         {
-          if (state_data->prog_data->args->common.flags & IPMI_FLAGS_DEBUG_DUMP)
-            fprintf(stderr,
-                    "ipmi_cmd_get_pef_configuration_parameters_alert_string: %s\n",
-                    ipmi_device_strerror(ipmi_device_errnum(state_data->dev)));
-          rv = CONFIG_ERR_NON_FATAL_ERROR;
+          if (state_data->prog_data->args->config_args.common.debug)
+            pstdout_fprintf(state_data->pstate,
+                            stderr,
+                            "ipmi_cmd_get_pef_configuration_parameters_alert_string: %s\n",
+                            ipmi_ctx_strerror(ipmi_ctx_errnum(state_data->ipmi_ctx)));
+          if (!IPMI_CTX_ERRNUM_IS_FATAL_ERROR(state_data->ipmi_ctx))
+            rv = CONFIG_ERR_NON_FATAL_ERROR;
           goto cleanup;
         }
       
@@ -240,11 +268,10 @@ alert_string_checkout (const char *section_name,
        * whatever is passed in, so don't check for overflow errors
        * from Fiid_obj_get_data.
        */
-      if (Fiid_obj_get_data (obj_cmd_rs,
-                             "string_data",
-                             alert_string + (i * 16),
-                             PEF_ALERT_STRING_MAX_LEN - (i * 16)) < 0)
-        goto cleanup;
+      _FIID_OBJ_GET_DATA (obj_cmd_rs,
+                          "string_data",
+                          alert_string + (i * 16),
+                          PEF_ALERT_STRING_MAX_LEN - (i * 16));
 
       /* Check if we've found a nul character */
       for (j = 0; j < 16; j++)
@@ -255,12 +282,14 @@ alert_string_checkout (const char *section_name,
     }
 
  done:
-  if (config_section_update_keyvalue_output(kv, (char *)alert_string) < 0)
+  if (config_section_update_keyvalue_output(state_data->pstate, 
+                                            kv, 
+                                            (char *)alert_string) < 0)
     return CONFIG_ERR_FATAL_ERROR;
 
   rv = CONFIG_ERR_SUCCESS;
  cleanup:
-  Fiid_obj_destroy(obj_cmd_rs);
+  _FIID_OBJ_DESTROY(obj_cmd_rs);
   return (rv);
 
 }
@@ -282,8 +311,7 @@ alert_string_commit (const char *section_name,
 
   string_selector = atoi (section_name + strlen ("Alert_String_"));
 
-  if (!(obj_cmd_rs = Fiid_obj_create(tmpl_cmd_set_pef_configuration_parameters_rs)))
-    goto cleanup;
+  _FIID_OBJ_CREATE(obj_cmd_rs, tmpl_cmd_set_pef_configuration_parameters_rs);
   
   alert_string_len = strlen((char *)kv->value_input);
 
@@ -292,7 +320,8 @@ alert_string_commit (const char *section_name,
   
   if (!(alert_string_buf = (uint8_t *)malloc(alert_string_buf_len)))
     {
-      perror("strdup");
+      pstdout_perror(state_data->pstate,
+                     "strdup");
       goto cleanup;
     }
   memset(alert_string_buf, '\0', alert_string_buf_len);
@@ -314,18 +343,20 @@ alert_string_commit (const char *section_name,
       else
         len_to_write = 16;
 
-      if (ipmi_cmd_set_pef_configuration_parameters_alert_strings (state_data->dev,
+      if (ipmi_cmd_set_pef_configuration_parameters_alert_strings (state_data->ipmi_ctx,
                                                                    string_selector,
                                                                    i+1,
                                                                    alert_string_buf + (i * 16),
                                                                    len_to_write,
                                                                    obj_cmd_rs) < 0)
         {
-          if (state_data->prog_data->args->common.flags & IPMI_FLAGS_DEBUG_DUMP)
-            fprintf(stderr,
-                    "ipmi_cmd_set_pef_configuration_parameters_alert_strings: %s\n",
-                    ipmi_device_strerror(ipmi_device_errnum(state_data->dev)));
-          rv = CONFIG_ERR_NON_FATAL_ERROR;
+          if (state_data->prog_data->args->config_args.common.debug)
+            pstdout_fprintf(state_data->pstate,
+                            stderr,
+                            "ipmi_cmd_set_pef_configuration_parameters_alert_strings: %s\n",
+                            ipmi_ctx_strerror(ipmi_ctx_errnum(state_data->ipmi_ctx)));
+          if (!IPMI_CTX_ERRNUM_IS_FATAL_ERROR(state_data->ipmi_ctx))
+            rv = CONFIG_ERR_NON_FATAL_ERROR;
           goto cleanup;
         }
     }
@@ -334,7 +365,7 @@ alert_string_commit (const char *section_name,
  cleanup:
   if (alert_string_buf)
     free(alert_string_buf);
-  Fiid_obj_destroy(obj_cmd_rs);
+  _FIID_OBJ_DESTROY(obj_cmd_rs);
   return (rv);
 
 }
@@ -342,7 +373,8 @@ alert_string_commit (const char *section_name,
 static config_validate_t
 alert_string_validate (const char *section_name,
                        const char *key_name,
-                       const char *value)
+                       const char *value,
+                       void *arg)
 {
   if (strlen (value) <= PEF_ALERT_STRING_MAX_LEN)
     return CONFIG_VALIDATE_VALID_VALUE;
@@ -353,23 +385,29 @@ struct config_section *
 pef_config_alert_string_section_get (pef_config_state_data_t *state_data, int num)
 {
   struct config_section *section = NULL;
-  char buf[64];
+  char buf[CONFIG_MAX_SECTION_NAME_LEN];
 
   if (num <= 0)
     {
-      fprintf(stderr, "Invalid Num = %d\n", num);
+      pstdout_fprintf(state_data->pstate,
+                      stderr, 
+                      "Invalid Num = %d\n", num);
       return NULL;
     }
 
-  snprintf(buf, 64, "Alert_String_%d", num);
+  snprintf(buf, CONFIG_MAX_SECTION_NAME_LEN, "Alert_String_%d", num);
 
-  if (!(section = config_section_create (buf, 
+  if (!(section = config_section_create (state_data->pstate, 
+                                         buf, 
                                          NULL, 
                                          NULL, 
-                                         0)))
+                                         0,
+                                         NULL,
+                                         NULL)))
     goto cleanup;
 
-  if (config_section_add_key (section,
+  if (config_section_add_key (state_data->pstate, 
+                              section,
                               "Event_Filter_Number",
                               "Give valid number",
                               0,
@@ -378,7 +416,8 @@ pef_config_alert_string_section_get (pef_config_state_data_t *state_data, int nu
                               config_number_range_seven_bits) < 0) 
     goto cleanup;
 
-  if (config_section_add_key (section,
+  if (config_section_add_key (state_data->pstate, 
+                              section,
                               "Alert_String_Set",
                               "Give valid number",
                               0,
@@ -387,7 +426,8 @@ pef_config_alert_string_section_get (pef_config_state_data_t *state_data, int nu
                               config_number_range_seven_bits) < 0) 
     goto cleanup;
 
-  if (config_section_add_key (section,
+  if (config_section_add_key (state_data->pstate, 
+                              section,
                               "Alert_String",
                               "Give string. Max 64 chars.",
                               0,
@@ -400,7 +440,7 @@ pef_config_alert_string_section_get (pef_config_state_data_t *state_data, int nu
 
  cleanup:
   if (section)
-    config_section_destroy(section);
+    config_section_destroy(state_data->pstate, section);
   return NULL;
 }
 
